@@ -34,11 +34,14 @@
  *      hangs in the fread loop instead of keeping state and returning.
  *      may or may not work on a little-endian machine
  */
+
+#define __USE_POSIX199309 1
+#include <time.h> // for nanosleep()
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 
-#define __USE_GNU
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
@@ -57,8 +60,8 @@
 
 #ifdef GARMIN_ENABLE
 
-#define GARMIN_LAYERID_TRANSPORT  0
-#define GARMIN_LAYERID_APPL      20
+#define GARMIN_LAYERID_TRANSPORT (unsigned char)  0
+#define GARMIN_LAYERID_APPL      (unsigned int) 20
 // Linux Garmin USB driver layer-id to use for some control mechanisms
 #define GARMIN_LAYERID_PRIVATE  0x01106E4B
 
@@ -90,6 +93,8 @@
 
 #define MAX_BUFFER_SIZE 4096
 
+#define GARMIN_CHANNELS	12
+
 // something magic about 64, garmin driver will not return more than
 // 64 at a time.  If you read less than 64 bytes the next read will
 // just get the last of the 64 byte buffer.
@@ -111,7 +116,7 @@ typedef struct {
 /* Garmin D800_Pvt_Date_Type */
 // This is the data format of the position data from the garmin USB
 typedef struct {
-	float alt;  /* ->fix.altitude above WGS 84 (meters) */
+	float alt;  /* altitude above WGS 84 (meters) */
 	float epe;  /* estimated position error, 2 sigma (meters)  */
 	float eph;  /* epe, but horizontal only (meters) */
 	float epv;  /* epe but vertical only (meters ) */
@@ -133,6 +138,7 @@ typedef struct {
 	long	grmn_days;
 } cpo_pvt_data;
 
+#ifdef __UNUSED__
 typedef struct {
 	unsigned long cycles;
 	double	 pr;
@@ -146,9 +152,9 @@ typedef struct {
 typedef struct {
 	double rcvr_tow;
 	short	rcvr_wn;
-	cpo_rcv_sv_data sv[MAXCHANNELS];
+	cpo_rcv_sv_data sv[GARMIN_CHANNELS];
 } cpo_rcv_data;
-
+#endif /* __UNUSED__ */
 
 // This is the packet format to/from the Garmin USB
 typedef struct {
@@ -159,7 +165,8 @@ typedef struct {
     unsigned short mReserved3;
     unsigned long  mDataSize;
     union {
-	    char  chars[MAX_BUFFER_SIZE];
+	    char chars[MAX_BUFFER_SIZE];
+	    unsigned char uchars[MAX_BUFFER_SIZE];
             cpo_pvt_data pvt;
             cpo_sat_data sats;
     } mData;
@@ -167,69 +174,70 @@ typedef struct {
 
 // useful funcs to read/write ints, only tested on Intel byte order
 //  floats and doubles are Intel order only...
-static inline void set_int(unsigned char *buf, int value)
+static inline void set_int(unsigned char *buf, unsigned int value)
 {
-        buf[0] = 0xFF & value;
-        buf[1] = 0xFF & (value >> 8);
-        buf[2] = 0xFF & (value >> 16);
-        buf[3] = 0xFF & (value >> 24);
+        buf[0] = (unsigned char)(0x0FF & value);
+        buf[1] = (unsigned char)(0x0FF & (value >> 8));
+        buf[2] = (unsigned char)(0x0FF & (value >> 16));
+        buf[3] = (unsigned char)(0x0FF & (value >> 24));
 }
 
-static inline int get_short(const unsigned char *buf)
+static inline unsigned short get_short(const unsigned char *buf)
 {
-        return  (0xFF & buf[0]) | ((0xFF & buf[1]) << 8);
+        return  (unsigned short)(0xFF & buf[0]) 
+		| (unsigned short)((0xFF & buf[1]) << 8);
 }
 
-static inline int get_int(const unsigned char *buf)
+static inline unsigned int get_int(const unsigned char *buf)
 {
-        return  (0xFF & buf[0]) | ((0xFF & buf[1]) << 8) | ((0xFF & buf[2]) << 16) | ((0xFF & buf[3]) << 24);
+        return  (unsigned int)(0xFF & buf[0]) | ((0xFF & buf[1]) << 8) | ((0xFF & buf[2]) << 16) | ((0xFF & buf[3]) << 24);
 }
 
 // convert radians to degrees
 static inline double  radtodeg( double rad) {
-	return ( rad * RAD_2_DEG );
+	return (double)(rad * RAD_2_DEG );
 }
 
-static int PrintPacket(struct gps_device_t *session, Packet_t *pkt );
+static gps_mask_t PrintPacket(struct gps_device_t *session, Packet_t *pkt );
 static void SendPacket (struct gps_device_t *session, Packet_t *aPacket );
 static int GetPacket (struct gps_device_t *session );
 
 // For debugging, decodes and prints some known packets.
-static int PrintPacket(struct gps_device_t *session, Packet_t *pkt)
+static gps_mask_t PrintPacket(struct gps_device_t *session, Packet_t *pkt)
 {
-    int mask = 0;
+    gps_mask_t mask = 0;
     int maj_ver;
     int min_ver;
-    int mode;
-    int prod_id;
-    int ver;
+    unsigned int mode;
+    unsigned short prod_id;
+    unsigned short ver;
+    unsigned int veri;
     time_t time_l = 0;
     unsigned int serial;
     cpo_sat_data *sats = NULL;
     cpo_pvt_data *pvt = NULL;
-    char buf[BUFSIZ] = "", *bufp = buf;
-    unsigned int i = 0, j = 0;
+    int i = 0, j = 0;
     double track;
 
-    gpsd_report(3, "PrintPacket() ");
+    gpsd_report(3, "PrintPacket()\n");
     if ( 4096 < pkt->mDataSize) {
 	gpsd_report(3, "bogus packet, size too large=%d\n", pkt->mDataSize);
 	return 0;
     }
 
-    snprintf(session->gpsdata.tag, sizeof(session->gpsdata.tag), "%d", pkt->mPacketType);
+    (void)snprintf(session->gpsdata.tag, sizeof(session->gpsdata.tag), "%u"
+	, (unsigned int)pkt->mPacketType);
     switch ( pkt->mPacketType ) {
     case GARMIN_LAYERID_TRANSPORT:
-	gpsd_report(3, "Transport ");
 	switch( pkt->mPacketId ) {
 	case GARMIN_PKTID_TRANSPORT_START_SESSION_REQ:
-	    gpsd_report(3, "Start Session req\n");
+	    gpsd_report(3, "Transport, Start Session req\n");
 	    break;
 	case GARMIN_PKTID_TRANSPORT_START_SESSION_RESP:
-	    gpsd_report(3, "Start Session resp\n");
+	    gpsd_report(3, "Transport, Start Session resp\n");
 	    break;
 	default:
-	    gpsd_report(3, "Packet: Type %d %d %d, ID: %d, Sz: %d\n"
+	    gpsd_report(3, "Transport, Packet: Type %d %d %d, ID: %d, Sz: %d\n"
 			, pkt->mPacketType
 			, pkt->mReserved1
 			, pkt->mReserved2
@@ -239,17 +247,16 @@ static int PrintPacket(struct gps_device_t *session, Packet_t *pkt)
 	}
 	break;
     case GARMIN_LAYERID_APPL:
-	gpsd_report(3, "Appl ");
 	switch( pkt->mPacketId ) {
 	case GARMIN_PKTID_PRODUCT_RQST:
-	    gpsd_report(3, "Product Data req\n");
+	    gpsd_report(3, "Appl, Product Data req\n");
 	    break;
 	case GARMIN_PKTID_PRODUCT_DATA:
-	    prod_id = get_short(&pkt->mData.chars[0]);
-	    ver = get_short(&pkt->mData.chars[2]);
+	    prod_id = get_short(&pkt->mData.uchars[0]);
+	    ver = get_short(&pkt->mData.uchars[2]);
 	    maj_ver = ver / 100;
 	    min_ver = ver - (maj_ver * 100);
-	    gpsd_report(3, "Product Data, sz: %d\n"
+	    gpsd_report(3, "Appl, Product Data, sz: %d\n"
 			, pkt->mDataSize);
 	    gpsd_report(1, "Garmin Product ID: %d, SoftVer: %d.%02d\n"
 			, prod_id, maj_ver, min_ver);
@@ -257,50 +264,49 @@ static int PrintPacket(struct gps_device_t *session, Packet_t *pkt)
 			, &pkt->mData.chars[4]);
 	    break;
 	case GARMIN_PKTID_PVT_DATA:
-	    gpsd_report(3, "PVT Data Sz: %d\n", pkt->mDataSize);
+	    gpsd_report(3, "Appl, PVT Data Sz: %d\n", pkt->mDataSize);
 
 	    pvt = &pkt->mData.pvt;
 
 	    // 631065600, unix seconds for 31 Dec 1989 Zulu 
-	    time_l = 631065600 + (pvt->grmn_days * 86400);
+	    time_l = (time_t)(631065600 + (pvt->grmn_days * 86400));
 	    time_l -= pvt->leap_sec;
 	    session->context->leap_seconds = pvt->leap_sec;
 	    session->context->valid = LEAP_SECOND_VALID;
 	    // gps_tow is always like x.999 or x.998 so just round it
-	    time_l += (time_t) rint(pvt->gps_tow);
-	    session->gpsdata.fix.time = session->gpsdata.sentence_time = time_l;
-#ifdef NTPSHM_ENABLE
-	    ntpshm_put(session, session->gpsdata.fix.time);
-#endif 
-	    gpsd_report(5, "time_l: %ld\n", time_l);
+	    time_l += (time_t) round(pvt->gps_tow);
+	    session->gpsdata.newdata.time 
+		= session->gpsdata.sentence_time 
+		= (double)time_l;
+	    gpsd_report(5, "time_l: %ld\n", (long int)time_l);
 
-	    session->gpsdata.fix.latitude = radtodeg(pvt->lat);
-	    session->gpsdata.fix.longitude = radtodeg(pvt->lon);
+	    session->gpsdata.newdata.latitude = radtodeg(pvt->lat);
+	    session->gpsdata.newdata.longitude = radtodeg(pvt->lon);
 
 	    // altitude over WGS84 converted to MSL
-	    session->gpsdata.fix.altitude = pvt->alt + pvt->msl_hght;
+	    session->gpsdata.newdata.altitude = pvt->alt + pvt->msl_hght;
 
 	    // geoid separation from WGS 84
             // gpsd sign is opposite of garmin sign
-	    session->separation = -pvt->msl_hght;
+	    session->gpsdata.separation = -pvt->msl_hght;
 
 	    // estimated position error in meters
 	    session->gpsdata.epe = pvt->epe;
-	    session->gpsdata.fix.eph = pvt->eph;
-	    session->gpsdata.fix.epv = pvt->epv;
+	    session->gpsdata.newdata.eph = pvt->eph;
+	    session->gpsdata.newdata.epv = pvt->epv;
 
 	    // convert lat/lon to knots
-	    session->gpsdata.fix.speed
+	    session->gpsdata.newdata.speed
 		= hypot(pvt->lon_vel, pvt->lat_vel) * 1.9438445;
 
             // keep climb in meters/sec
-	    session->gpsdata.fix.climb = pvt->alt_vel;
+	    session->gpsdata.newdata.climb = pvt->alt_vel;
 
 	    track = atan2(pvt->lon_vel, pvt->lat_vel);
 	    if (track < 0) {
 		track += 2 * PI;
 	    }
-	    session->gpsdata.fix.track = radtodeg(track);
+	    session->gpsdata.newdata.track = radtodeg(track);
 
 	    switch ( pvt->fix) {
 	    case 0:
@@ -308,39 +314,43 @@ static int PrintPacket(struct gps_device_t *session, Packet_t *pkt)
 	    default:
 		// no fix
 		session->gpsdata.status = STATUS_NO_FIX;
-		session->gpsdata.fix.mode = MODE_NO_FIX;
+		session->gpsdata.newdata.mode = MODE_NO_FIX;
 		break;
 	    case 2:
 		// 2D fix
 		session->gpsdata.status = STATUS_FIX;
-		session->gpsdata.fix.mode = MODE_2D;
+		session->gpsdata.newdata.mode = MODE_2D;
 		break;
 	    case 3:
 		// 3D fix
 		session->gpsdata.status = STATUS_FIX;
-		session->gpsdata.fix.mode = MODE_3D;
+		session->gpsdata.newdata.mode = MODE_3D;
 		break;
 	    case 4:
 		// 2D Differential fix
 		session->gpsdata.status = STATUS_DGPS_FIX;
-		session->gpsdata.fix.mode = MODE_2D;
+		session->gpsdata.newdata.mode = MODE_2D;
 		break;
 	    case 5:
 		// 3D differential fix
 		session->gpsdata.status = STATUS_DGPS_FIX;
-		session->gpsdata.fix.mode = MODE_3D;
+		session->gpsdata.newdata.mode = MODE_3D;
 		break;
 	    }
+#ifdef NTPSHM_ENABLE
+	    if (session->gpsdata.newdata.mode > MODE_NO_FIX)
+		(void) ntpshm_put(session, session->gpsdata.newdata.time);
+#endif /* NTPSHM_ENABLE */
 
-	    gpsd_report(4, "mode %d, status %d\n"
-			, session->gpsdata.fix.mode
+	    gpsd_report(4, "Appl, mode %d, status %d\n"
+			, session->gpsdata.newdata.mode
 			, session->gpsdata.status);
 
-	    gpsd_report(3, "UTC Time: %lf\n", session->gpsdata.fix.time);
+	    gpsd_report(3, "UTC Time: %lf\n", session->gpsdata.newdata.time);
 	    gpsd_report(3, "Geoid Separation (MSL - WGS84): from garmin %lf, calculated %lf\n"
 		, -pvt->msl_hght
-		, wgs84_separation(session->gpsdata.fix.latitude
-			, session->gpsdata.fix.longitude));
+		, wgs84_separation(session->gpsdata.newdata.latitude
+			, session->gpsdata.newdata.longitude));
 	    gpsd_report(3, "Alt: %.3f, Epe: %.3f, Eph: %.3f, Epv: %.3f, Fix: %d, Gps_tow: %f, Lat: %.3f, Lon: %.3f, LonVel: %.3f, LatVel: %.3f, AltVel: %.3f, MslHgt: %.3f, Leap: %d, GarminDays: %ld\n"
 			, pvt->alt
 			, pvt->epe
@@ -348,8 +358,8 @@ static int PrintPacket(struct gps_device_t *session, Packet_t *pkt)
 			, pvt->epv
 			, pvt->fix
 			, pvt->gps_tow
-			, session->gpsdata.fix.latitude
-			, session->gpsdata.fix.longitude
+			, session->gpsdata.newdata.latitude
+			, session->gpsdata.newdata.longitude
 			, pvt->lon_vel
 			, pvt->lat_vel
 			, pvt->alt_vel
@@ -357,19 +367,16 @@ static int PrintPacket(struct gps_device_t *session, Packet_t *pkt)
 			, pvt->leap_sec
 			, pvt->grmn_days);
 
-	    gpsd_binary_fix_dump(session, bufp);
-	    bufp += strlen(bufp);
-	    mask |= TIME_SET | LATLON_SET | ALTITUDE_SET | STATUS_SET | MODE_SET | SPEED_SET | TRACK_SET | CLIMB_SET | HERR_SET | VERR_SET | PERR_SET;
+	    mask |= TIME_SET | LATLON_SET | ALTITUDE_SET | STATUS_SET | MODE_SET | SPEED_SET | TRACK_SET | CLIMB_SET | HERR_SET | VERR_SET | PERR_SET | CYCLE_START_SET;
 	    break;
 	case GARMIN_PKTID_SAT_DATA:
-	    gpsd_report(3, "SAT Data Sz: %d\n", pkt->mDataSize);
+	    gpsd_report(3, "Appl, SAT Data Sz: %d\n", pkt->mDataSize);
 	    sats = &pkt->mData.sats;
 
 	    session->gpsdata.satellites_used = 0;
+	    memset(session->gpsdata.used,0,sizeof(session->gpsdata.used));
 	    gpsd_zero_satellites(&session->gpsdata);
-	    for ( i = 0 ; i < MAXCHANNELS ; i++ )
-                session->gpsdata.used[i] = 0;
-	    for ( i = 0, j = 0 ; i < MAXCHANNELS ; i++, sats++ ) {
+	    for ( i = 0, j = 0 ; i < GARMIN_CHANNELS ; i++, sats++ ) {
 		gpsd_report(4,
 			    "  Sat %d, snr: %d, elev: %d, Azmth: %d, Stat: %x\n"
 			    , sats->svid
@@ -380,46 +387,40 @@ static int PrintPacket(struct gps_device_t *session, Packet_t *pkt)
 
 		// busted??
 // FIXME!! need to elimate sats over 32
-		if ( (32 < sats->svid) || (0 == sats->svid) ) {
+		if ( (32 < (int)sats->svid) || (0 == (int)sats->svid) ) {
 		    // Garmin uses 255 for empty
 		    // gpsd uses 0 for empty
 		    continue;
 		}
 
-		session->gpsdata.PRN[j] = sats->svid;
-		session->gpsdata.azimuth[j] = sats->azmth;
-		session->gpsdata.elevation[j] = sats->elev;
+		session->gpsdata.PRN[j]       = (int)sats->svid;
+		session->gpsdata.azimuth[j]   = (int)sats->azmth;
+		session->gpsdata.elevation[j] = (int)sats->elev;
 		// snr units??
 		// garmin 0 -> 0xffff, NMEA 99 -> 0
 		session->gpsdata.ss[j]
-		    = 99 - ((100 *(long)sats->snr) >> 16);
-		if ( sats->status & 4 ) {
+		    = 99 - (int)((100 *( unsigned long)sats->snr) >> 16);
+		if ( (unsigned char)0 != (sats->status & 4 ) )  {
 		    // used in solution?
-		    session->gpsdata.used[ session->gpsdata.satellites_used++] = sats->svid;
+		    session->gpsdata.used[session->gpsdata.satellites_used++]
+			= (int)sats->svid;
 		}
 		session->gpsdata.satellites++;
 		j++;
 	    }
 	    mask |= SATELLITE_SET;
-	    bufp += strlen(bufp);
-	    // dump NMEA GPGSA, PGRME
-	    gpsd_binary_quality_dump(session, bufp+strlen(bufp));
-	    bufp += strlen(bufp);
-	    // dump NMEA GPGSV
-	    gpsd_binary_satellite_dump(session, bufp+strlen(bufp));
-	    bufp += strlen(bufp);
 	    break;
 	case GARMIN_PKTID_PROTOCOL_ARRAY:
             // this packet is never requested, it just comes, in some case
             // after a GARMIN_PKTID_PRODUCT_RQST 
-	    gpsd_report(3, "Product Capability: %d\n", pkt->mDataSize);
-            for ( i = 0; i < pkt->mDataSize ; i += 3 ) {
+	    gpsd_report(3, "Appl, Product Capability: %d\n", pkt->mDataSize);
+            for ( i = 0; i < (int)pkt->mDataSize ; i += 3 ) {
 		    gpsd_report(3, "  %c%03d\n", pkt->mData.chars[i]
-			, get_short( &(pkt->mData.chars[i+1])) );
+			, get_short( &(pkt->mData.uchars[i+1])) );
 	    }
 	    break;
 	default:
-	    gpsd_report(3, "ID: %d, Sz: %d\n"
+	    gpsd_report(3, "Appl, ID: %d, Sz: %d\n"
 			, pkt->mPacketId
 			, pkt->mDataSize);
 	    break;
@@ -427,23 +428,22 @@ static int PrintPacket(struct gps_device_t *session, Packet_t *pkt)
 	break;
     case 75:
 	// private
-	gpsd_report(3, "Private ");
 	switch( pkt->mPacketId ) {
 	case PRIV_PKTID_INFO_REQ:
-	    gpsd_report(3, "ID: Info Req\n");
+	    gpsd_report(3, "Private, ID: Info Req\n");
 	    break;
 	case PRIV_PKTID_INFO_RESP:
-	    ver = get_int(pkt->mData.chars);
-	    maj_ver = ver >> 16;
-	    min_ver = ver & 0xffff;
-	    mode = get_int(&pkt->mData.chars[4]);
-	    serial = get_int(&pkt->mData.chars[8]);
-	    gpsd_report(3, "ID: Info Resp\n");
+	    veri = get_int(pkt->mData.uchars);
+	    maj_ver = (int)(veri >> 16);
+	    min_ver = (int)(veri & 0xffff);
+	    mode = get_int(&pkt->mData.uchars[4]);
+	    serial = get_int(&pkt->mData.uchars[8]);
+	    gpsd_report(3, "Private, ID: Info Resp\n");
 	    gpsd_report(1, "Garmin USB Driver found, Version %d.%d, Mode: %d, GPS Serial# %u\n"
 			,  maj_ver, min_ver, mode, serial);
 	    break;
 	default:
-	    gpsd_report(3, "Packet: ID: %d, Sz: %d\n"
+	    gpsd_report(3, "Private, Packet: ID: %d, Sz: %d\n"
 			, pkt->mPacketId
 			, pkt->mDataSize);
 	    break;
@@ -459,10 +459,6 @@ static int PrintPacket(struct gps_device_t *session, Packet_t *pkt)
 	break;
     }
 
-    if ( bufp != buf ) {
-	gpsd_report(3, "%s", buf);
-    }
-
     return mask;
 }
 
@@ -470,11 +466,11 @@ static int PrintPacket(struct gps_device_t *session, Packet_t *pkt)
 // send a packet in GarminUSB format
 static void SendPacket (struct gps_device_t *session, Packet_t *aPacket ) 
 {
-	long theBytesToWrite = 12 + aPacket->mDataSize;
-	long theBytesReturned = 0;
+	size_t theBytesToWrite = (size_t)(12 + aPacket->mDataSize);
+	ssize_t theBytesReturned = 0;
 
         gpsd_report(4, "SendPacket(), writing %d bytes\n", theBytesToWrite);
-        PrintPacket ( session,  aPacket);
+        (void)PrintPacket ( session,  aPacket);
 
 	theBytesReturned = write( session->gpsdata.gps_fd
 		    , aPacket, theBytesToWrite);
@@ -518,8 +514,9 @@ static int GetPacket (struct gps_device_t *session )
     int cnt = 0;
     // int x = 0; // for debug dump
 
-    memset( session->GarminBuffer, 0, sizeof(Packet_t));
-    session->GarminBufferLen = 0;
+    memset( session->garmin.Buffer, 0, sizeof(Packet_t));
+    memset( &delay, 0, sizeof(delay));
+    session->garmin.BufferLen = 0;
 
     gpsd_report(4, "GetPacket()\n");
 
@@ -529,11 +526,11 @@ static int GetPacket (struct gps_device_t *session )
 
 	// not optimal, but given the speed and packet nature of
 	// the USB not too bad for a start
-	long theBytesReturned = 0;
-	char *buf = (char *)session->GarminBuffer;
+	ssize_t theBytesReturned = 0;
+	unsigned char *buf = session->garmin.Buffer;
 
 	theBytesReturned = read(session->gpsdata.gps_fd
-		, buf + session->GarminBufferLen
+		, buf + session->garmin.BufferLen
 		, ASYNC_DATA_SIZE);
         if ( 0 >  theBytesReturned ) {
 	    // read error...
@@ -544,35 +541,36 @@ static int GetPacket (struct gps_device_t *session )
 	}
 	gpsd_report(5, "got %d bytes\n", theBytesReturned);
 
-	session->GarminBufferLen += theBytesReturned;
+	session->garmin.BufferLen += theBytesReturned;
 	if ( 64 > theBytesReturned ) {
 	    // zero length, or short, read is a flag for got the whole packet
             break;
 	}
 		
-	if ( 256 <=  session->GarminBufferLen ) {
+	if ( 256 <=  session->garmin.BufferLen ) {
 	    // really bad read error...
-	    session->GarminBufferLen = 0;
+	    session->garmin.BufferLen = 0;
 	    gpsd_report(3, "GetPacket() packet too long!\n");
 	    break;
 	}
 
+	/*@ ignore @*/
 	delay.tv_sec = 0;
 	delay.tv_nsec = 3330000L;
 	while (nanosleep(&delay, &rem) < 0)
 	    continue;
-
+	/*@ end @*/
     }
     // dump the individual bytes, debug only
-    // for ( x = 0; x < session->GarminBufferLen; x++ ) {
-        // gpsd_report(6, "p[%d] = %x\n", x, session->GarminBuffer[x]);
+    // for ( x = 0; x < session->garmin.BufferLen; x++ ) {
+        // gpsd_report(6, "p[%d] = %x\n", x, session->garmin.Buffer[x]);
     // }
     if ( 10 <= cnt ) {
 	    gpsd_report(3, "GetPacket() packet too long or too slow!\n");
 	    return -1;
     }
 
-    gpsd_report(5, "GotPacket() sz=%d \n", session->GarminBufferLen);
+    gpsd_report(5, "GotPacket() sz=%d \n", session->garmin.BufferLen);
     return 0;
 }
 
@@ -582,68 +580,51 @@ static int GetPacket (struct gps_device_t *session )
  * return 1 if garmin_gps device found
  * return 0 if not
  */
-static int garmin_probe(struct gps_device_t *session)
+static bool garmin_probe(struct gps_device_t *session)
 {
 
     Packet_t *thePacket = NULL;
-    char *buffer = NULL;
-    char str_buf[256];
+    unsigned char *buffer = NULL;
     fd_set fds, rfds;
     struct timeval tv;
     int sel_ret = 0;
     int ok = 0;
-    FILE *fp;
     int i;
 
-    // check for USB serial drivers
-    // very Linux specific
-    if ((fp = fopen( "/proc/tty/driver/usbserial", "r")) == NULL) {
-	gpsd_report(2, "No USB serial drivers found.\n");
-	return 0;
-    } else {
-        // try to find garmin_gps driver
-	while ( fgets( str_buf, sizeof(str_buf), fp) ) {
-		// early garmin driver: garmin_gps
-	        // later garmin driver: Garmin USB/TTY
-		if ( strcasestr( str_buf, "garmin") ) {
-			// yes, the garmin_gps driver is active
-			ok = 1;
-			break;
-		}
-	}
-    }
-    (void) fclose(fp);
-    if ( ! ok ) {
-	gpsd_report(5, "garmin_gps not active.\n");
-	return 0;
+    /* check for USB serial drivers -- very Linux-specific */
+    if (access("/sys/module/garmin_gps", R_OK) != 0) {
+	gpsd_report(5, "garmin_gps not active.\n"); 
+        return false;
     }
 
     /* Save original terminal parameters */
     if (tcgetattr(session->gpsdata.gps_fd,&session->ttyset_old) != 0) {
 	gpsd_report(0, "garmin_probe: error getting port attributes: %s\n",
              strerror(errno));
-	return 0;
+	return false;
     }
     memcpy(&session->ttyset,&session->ttyset_old,sizeof(session->ttyset));
 
-    cfmakeraw(&session->ttyset);
+    (void)cfmakeraw(&session->ttyset);
 
     if (tcsetattr( session->gpsdata.gps_fd, TCIOFLUSH, &session->ttyset) < 0) {
 	gpsd_report(0, "garmin_probe: error changing port attributes: %s\n",
              strerror(errno));
-	return 0;
+	return false;
     }
 
-    if ( !session->GarminBuffer ) {
-	    // get a packet buffer
-	    session->GarminBuffer = calloc( sizeof(Packet_t), 1);
-	    if ( !session->GarminBuffer ) {
-		gpsd_report(0, "garmin_probe: out of memory!\n");
-		return 0;
-            }
+    /* reset the buffer and buffer length */
+    memset( session->garmin.Buffer, 0, sizeof(session->garmin.Buffer) );
+    session->garmin.BufferLen = 0;
+
+    if (sizeof(session->garmin.Buffer) < sizeof(Packet_t)) {
+	gpsd_report(0, "garmin_probe: Compile error, garmin.Buffer too small.\n",
+             strerror(errno));
+	return false;
     }
-    thePacket = (Packet_t*)session->GarminBuffer;
-    buffer = (char *)thePacket;
+
+    buffer = session->garmin.Buffer;
+    thePacket = (Packet_t*)buffer;
 
     // set Mode 0
     set_int(buffer, GARMIN_LAYERID_PRIVATE);
@@ -671,6 +652,7 @@ static int garmin_probe(struct gps_device_t *session)
     // Wait, nicely, until the device returns the Version info
     // Toss any other packets, up to 4
     ok = 0;
+    memset( &tv,0,sizeof(tv));
     for( i = 0 ; i < 4 ; i++ ) {
         memcpy((char *)&rfds, (char *)&fds, sizeof(rfds));
 
@@ -680,17 +662,18 @@ static int garmin_probe(struct gps_device_t *session)
 	    if (errno == EINTR)
 		continue;
 	    gpsd_report(0, "select: %s\n", strerror(errno));
-	    return(0);
+	    return false;
 	} else if ( sel_ret == 0 ) {
 	    gpsd_report(3, "garmin_probe() timeout\n");
 	    // restore old terminal settings
-            tcsetattr(session->gpsdata.gps_fd, TCIOFLUSH, &session->ttyset_old);
-	    return(0);
+            (void)tcsetattr(session->gpsdata.gps_fd, TCIOFLUSH
+		, &session->ttyset_old);
+	    return false;
         }
-	if ( !GetPacket( session ) ) {
-	    PrintPacket(session, thePacket);
+	if ( 0 == GetPacket( session ) ) {
+	    (void)PrintPacket(session, thePacket);
 
-	    if( ( 75 == thePacket->mPacketType)
+	    if( ( (unsigned char)75 == thePacket->mPacketType)
 	        && (PRIV_PKTID_INFO_RESP == thePacket->mPacketId) ) {
                 ok = 1;
 	        break;
@@ -698,11 +681,12 @@ static int garmin_probe(struct gps_device_t *session)
 	}
     }
 
-    if ( ! ok ) {
+    if ( 0 == ok ) {
 	gpsd_report(2, "Garmin driver never answered to INFO_REQ.\n");
 	// restore old terminal settings
-        tcsetattr(session->gpsdata.gps_fd, TCIOFLUSH, &session->ttyset_old);
-	return 0;
+        (void)tcsetattr(session->gpsdata.gps_fd, TCIOFLUSH
+		, &session->ttyset_old);
+	return false;
     }
     // Tell the device that we are starting a session.
     gpsd_report(3, "Send Garmin Start Session\n");
@@ -729,11 +713,12 @@ static int garmin_probe(struct gps_device_t *session)
 	} else if ( sel_ret == 0 ) {
 	    gpsd_report(3, "garmin_probe() timeout\n");
 	    // restore old terminal settings
-            tcsetattr(session->gpsdata.gps_fd, TCIOFLUSH, &session->ttyset_old);
+            (void)tcsetattr(session->gpsdata.gps_fd, TCIOFLUSH
+		, &session->ttyset_old);
 	    return(0);
         }
-	if ( !GetPacket( session ) ) {
-	    PrintPacket(session, thePacket);
+	if ( 0 == GetPacket( session ) ) {
+	    (void)PrintPacket(session, thePacket);
 
 	    if( (GARMIN_LAYERID_TRANSPORT == thePacket->mPacketType)
 	        && (GARMIN_PKTID_TRANSPORT_START_SESSION_RESP
@@ -743,11 +728,12 @@ static int garmin_probe(struct gps_device_t *session)
 	    }
 	}
     }
-    if ( ! ok ) {
+    if ( 0 == ok ) {
 	gpsd_report(2, "Garmin driver never answered to START_SESSION.\n");
 	// restore old terminal settings
-        tcsetattr(session->gpsdata.gps_fd, TCIOFLUSH, &session->ttyset_old);
-	return 0;
+        (void)tcsetattr(session->gpsdata.gps_fd, TCIOFLUSH
+		, &session->ttyset_old);
+	return false;
     }
 
     // Tell the device to send product data
@@ -771,17 +757,18 @@ static int garmin_probe(struct gps_device_t *session)
 	    if (errno == EINTR)
 		continue;
 	    gpsd_report(0, "select: %s\n", strerror(errno));
-	    return(0);
+	    return false;
 	} else if ( sel_ret == 0 ) {
 	    gpsd_report(3, "garmin_probe() timeout\n");
 	    // restore old terminal settings
-            tcsetattr(session->gpsdata.gps_fd, TCIOFLUSH, &session->ttyset_old);
-	    return(0);
+            (void)tcsetattr(session->gpsdata.gps_fd, TCIOFLUSH
+		, &session->ttyset_old);
+	    return false;
         }
-	if ( !GetPacket( session ) ) {
-	    PrintPacket(session, thePacket);
+	if ( 0 == GetPacket( session ) ) {
+	    (void)PrintPacket(session, thePacket);
 
-	    if( (GARMIN_LAYERID_APPL == thePacket->mPacketType)
+	    if( (GARMIN_LAYERID_APPL == (unsigned int)thePacket->mPacketType)
 	        && ( GARMIN_PKTID_PRODUCT_DATA == thePacket->mPacketId) ) {
     		ok = 1;
 	        break;
@@ -789,13 +776,14 @@ static int garmin_probe(struct gps_device_t *session)
 	}
     }
 
-    if ( ! ok ) {
+    if ( 0 == ok ) {
 	gpsd_report(2, "Garmin driver never answered to PRODUCT_DATA.\n");
 	// restore old terminal settings
-        tcsetattr(session->gpsdata.gps_fd, TCIOFLUSH, &session->ttyset_old);
-	return 0;
+        (void)tcsetattr(session->gpsdata.gps_fd, TCIOFLUSH
+		, &session->ttyset_old);
+	return false;
     }
-    return(1);
+    return true;
 }
 
 /*
@@ -810,13 +798,13 @@ static int garmin_probe(struct gps_device_t *session)
  */
 static void garmin_init(struct gps_device_t *session)
 {
-	Packet_t *thePacket = (Packet_t*)session->GarminBuffer;
-	char *buffer = (char *)thePacket;
-	int ret;
+	unsigned char *buffer = session->garmin.Buffer;
+	Packet_t *thePacket = (Packet_t*)buffer;
+	bool ret;
 
 	gpsd_report(5, "to garmin_probe()\n");
 	ret = garmin_probe( session );
-	gpsd_report(3, "from garmin_probe() = %d\n", ret);
+	gpsd_report(3, "from garmin_probe() = %d\n", (int)ret);
 
 	// turn on PVT data 49
 	gpsd_report(3, "Set Garmin to send reports every 1 second\n");
@@ -837,14 +825,14 @@ static void garmin_init(struct gps_device_t *session)
 	//SendPacket(session,  thePacket);
 }
 
-static int garmin_get_packet(struct gps_device_t *session, int waiting UNUSED)
+static ssize_t garmin_get_packet(struct gps_device_t *session) 
 {
-    return !GetPacket( session );
+    return (ssize_t)( 0 == GetPacket( session ) ? 1 : 0);
 }
 
-static int garmin_parse_input(struct gps_device_t *session)
+static gps_mask_t garmin_parse_input(struct gps_device_t *session)
 {
-    return PrintPacket(session, (Packet_t*)session->GarminBuffer);
+    return PrintPacket(session, (Packet_t*)session->garmin.Buffer);
 }
 
 /* this is everything we export */
