@@ -5,28 +5,80 @@
 #include <math.h>
 #include <string.h>
 #include <stdarg.h>
-#define __USE_XOPEN
 #include <time.h>
 
 #include "gpsd.h"
 
+#define MONTHSPERYEAR	12		/* months per calendar year */
+
+void gps_clear_fix(/*@ out @*/struct gps_fix_t *fixp)
+/* stuff a fix structure with recognizable out-of-band values */
+{
+    fixp->time = NAN;
+    fixp->mode = MODE_NOT_SEEN;
+    fixp->track = NAN;
+    fixp->speed = NAN;
+    fixp->climb = NAN;
+    fixp->altitude = NAN;
+    fixp->ept = NAN;
+    fixp->eph = NAN;
+    fixp->epv = NAN;
+    fixp->epd = NAN;
+    fixp->eps = NAN;
+    fixp->epc = NAN;
+}
+
+void gps_merge_fix(/*@ out @*/struct gps_fix_t *to,
+		   gps_mask_t transfer,
+		   /*@ in @*/struct gps_fix_t *from)
+/* merge new data into an old fix */
+{
+    if ((transfer & TIME_SET)!=0)
+	to->time = from->time;
+    if ((transfer & LATLON_SET)!=0) {
+	to->latitude = from->latitude;
+	to->longitude = from->longitude;
+    }
+    if ((transfer & MODE_SET)!=0)
+	to->mode = from->mode;
+    if ((transfer & ALTITUDE_SET)!=0)
+	to->altitude = from->altitude;
+    if ((transfer & TRACK_SET)!=0)
+	to->track = from->track;
+    if ((transfer & SPEED_SET)!=0)
+	to->speed = from->speed;
+    if ((transfer & CLIMB_SET)!=0)
+	to->climb = from->climb;
+    if ((transfer & TIMERR_SET)!=0)
+	to->ept = from->ept;
+    if ((transfer & HERR_SET)!=0)
+	to->eph = from->eph;
+    if ((transfer & VERR_SET)!=0)
+	to->epv = from->epv;
+    if ((transfer & SPEEDERR_SET)!=0)
+	to->eps = from->eps;
+    if ((transfer & CLIMBERR_SET)!=0)
+	to->epc = from->epc;
+}
+
 double timestamp(void) 
 {
     struct timeval tv; 
-    gettimeofday(&tv, NULL); 
-    return(tv.tv_sec + tv.tv_usec/1e6);
+    (void)gettimeofday(&tv, NULL); 
+    /*@i1@*/return(tv.tv_sec + tv.tv_usec*1e-6);
 }
 
 time_t mkgmtime(register struct tm *t)
 /* struct tm to seconds since Unix epoch */
 {
-    register unsigned short year;
+    register int year;
     register time_t result;
-    static const int cumdays[12] =
+    static const int cumdays[MONTHSPERYEAR] =
     {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};
 
-    year = 1900 + t->tm_year + t->tm_mon / 12;
-    result = (year - 1970) * 365 + cumdays[t->tm_mon % 12];
+    /*@ +matchanyintegral @*/
+    year = 1900 + t->tm_year + t->tm_mon / MONTHSPERYEAR;
+    result = (year - 1970) * 365 + cumdays[t->tm_mon % MONTHSPERYEAR];
     result += (year - 1968) / 4;
     result -= (year - 1900) / 100;
     result += (year - 1600) / 400;
@@ -37,10 +89,11 @@ time_t mkgmtime(register struct tm *t)
     result += t->tm_min;
     result *= 60;
     result += t->tm_sec;
+    /*@ -matchanyintegral @*/
     return (result);
 }
 
-double iso8601_to_unix(char *isotime)
+double iso8601_to_unix(/*@in@*/char *isotime)
 /* ISO8601 UTC to Unix UTC */
 {
     char *dp = NULL;
@@ -52,26 +105,28 @@ double iso8601_to_unix(char *isotime)
 	usec = strtod(dp, NULL);
     else
 	usec = 0;
-    return mkgmtime(&tm) + usec;
+    return (double)mkgmtime(&tm) + usec;
 }
 
-char *unix_to_iso8601(double fixtime, char *isotime)
+/*@observer@*/char *unix_to_iso8601(double fixtime, /*@ out @*/char isotime[], int len)
 /* Unix UTC time to ISO8601, no timezone adjustment */
 {
     struct tm when;
     double integral, fractional;
     time_t intfixtime;
-    int slen;
+    size_t slen;
 
     fractional = modf(fixtime, &integral);
     intfixtime = (time_t)integral;
-    gmtime_r(&intfixtime, &when);
+    (void)gmtime_r(&intfixtime, &when);
 
-    strftime(isotime, 28, "%Y-%m-%dT%H:%M:%S", &when);
+    (void)strftime(isotime, 28, "%Y-%m-%dT%H:%M:%S", &when);
     slen = strlen(isotime);
-    sprintf(isotime + slen, "%.1f", fractional);
-    memcpy(isotime+slen, isotime+slen+1, strlen(isotime+slen+1));
-    strcat(isotime, "Z");
+    (void)snprintf(isotime + slen, (size_t)len, "%.1f", fractional);
+    /*@ -aliasunique @*/
+    (void)memcpy(isotime+slen, isotime+slen+1, strlen(isotime+slen+1));
+    /*@ -aliasunique @*/
+    (void)strcat(isotime, "Z");
     return isotime;
 }
 
@@ -81,32 +136,31 @@ char *unix_to_iso8601(double fixtime, char *isotime)
  * rollover happened at 0000 22 August 1999.  Time-of-week is in seconds.
  *
  * This code copes with both conventional GPS weeks and the "extended"
- * 15-or-16-bit version with no wraparound that apperas in Zodiac
+ * 15-or-16-bit version with no wraparound that appears in Zodiac
  * chips and is supposed to appear in the Geodetic Navigation
  * Information (0x29) packet of SiRF chips.  Some SiRF firmware versions
  * (notably 231) actually ship the wrapped 10-bit week, despite what
  * the protocol reference claims.
  *
  * Note: This time will need to be corrected for leap seconds.
- * That's what the offset argument is for.
  */
 #define GPS_EPOCH	315964800		/* GPS epoch in Unix time */
 #define SECS_PER_WEEK	(60*60*24*7)		/* seconds per week */
 #define GPS_ROLLOVER	(1024*SECS_PER_WEEK)	/* rollover period */
 
-double gpstime_to_unix(int week, double tow, int offset)
+double gpstime_to_unix(int week, double tow)
 {
     double fixtime;
 
-    if (week >= GPS_ROLLOVER)
+    if (week >= 1024)
 	fixtime = GPS_EPOCH + (week * SECS_PER_WEEK) + tow;
     else {
 	time_t now, last_rollover;
-	time(&now);
+	(void)time(&now);
 	last_rollover = GPS_EPOCH+((now-GPS_EPOCH)/GPS_ROLLOVER)*GPS_ROLLOVER;
-	fixtime = last_rollover + (week * SECS_PER_WEEK) + tow;
+	/*@i@*/fixtime = last_rollover + (week * SECS_PER_WEEK) + tow;
     }
-    return fixtime + offset;
+    return fixtime;
 }
 
 #define Deg2Rad(n)	((n) * DEG_2_RAD)
@@ -151,7 +205,7 @@ double earth_distance(double lat1, double lon1, double lat2, double lon2)
     // a very small amount due to rounding errors in the preceding
     // calculations (this is prone to happen when the argument points
     // are very close together).  Thus we constrain it here.
-    if (abs(a) > 1) 
+    if (fabs(a) > 1) 
 	a = 1;
     else if (a < -1) 
 	a = -1;
@@ -205,15 +259,49 @@ First, each compute element P(i,j) of the 4x4 product A~*A.
 If S(k=1,k=n): f(...) is the sum of f(...) as k varies from 1 to n, then
 applying the definition of matrix product tells us: 
 
-P(i,j) = S(k=1,k=n): B(i, k) * A(k, j) + 1
+P(i,j) = S(k=1,k=n): B(i, k) * A(k, j)
 
 But because B is the transpose of A, this reduces to 
 
-P(i,j) = S(k=1,k=n): A(k, i) * A(k, j) + 1
+P(i,j) = S(k=1,k=n): A(k, i) * A(k, j)
+
+This is not, however, the entire algorithm that SiRF uses.  Carl writes:
+
+> As you note, with rounding accounted for, most values agree exactly, and
+> those that don't agree our number is higher.  That is because we
+> deweight some satellites and account for that in the DOP calculation.
+> If a satellite is not used in a solution at the same weight as others,
+> it should not contribute to DOP calculation at the same weight.  So our
+> internal algorithm does a compensation for that which you would have no
+> way to duplicate on the outside since we don't output the weighting
+> factors.  In fact those are not even available to API users.
+
+Queried about the deweighting, Carl says:
+
+> In the SiRF tracking engine, each satellite track is assigned a quality
+> value based on the tracker's estimate of that signal.  It includes C/No
+> estimate, ability to hold onto the phase, stability of the I vs. Q phase
+> angle, etc.  The navigation algorithm then ranks all the tracks into
+> quality order and selects which ones to use in the solution and what
+> weight to give those used in the solution.  The process is actually a
+> bit of a "trial and error" method -- we initially use all available
+> tracks in the solution, then we sequentially remove the lowest quality
+> ones until the solution stabilizes.  The weighting is inherent in the
+> Kalman filter algorithm.  Once the solution is stable, the DOP is
+> computed from those SVs used, and there is an algorithm that looks at
+> the quality ratings and determines if we need to deweight any.
+> Likewise, if we use altitude hold mode for a 3-SV solution, we deweight
+> the phantom satellite at the center of the Earth.
+
+So we cannot exactly duplicate what SiRF does internally.  We'll leave
+HDOP alone and use our computed values for VDOP and PDOP.  Note, this
+may have to change in the future if this code is used by a non-SiRF
+driver.
 
 ******************************************************************************/
 
-static int invert(double mat[4][4], double inverse[4][4])
+/*@ -fixedformalarray -mustdefine @*/
+static int invert(double mat[4][4], /*@out@*/double inverse[4][4])
 {
   // Find all NECESSARY 2x2 subdeterminants
   //double Det2_12_01 = mat[1][0]*mat[2][1] - mat[1][1]*mat[2][0];
@@ -302,23 +390,26 @@ static int invert(double mat[4][4], double inverse[4][4])
 
   return 1;
 }  
+/*@ +fixedformalarray +mustdefine @*/
 
-void dop(int channels, struct gps_data_t *gpsdata)
+gps_mask_t dop(struct gps_data_t *gpsdata)
 {
     double prod[4][4];
     double inv[4][4];
     double satpos[MAXCHANNELS][4];
     int i, j, k, n;
 
+#ifdef __UNUSED__
     gpsd_report(0, "Satellite picture:\n");
     for (k = 0; k < MAXCHANNELS; k++) {
 	if (gpsdata->used[k])
 	    gpsd_report(0, "az: %d el: %d  SV: %d\n",
 			gpsdata->azimuth[k], gpsdata->elevation[k], gpsdata->used[k]);
     }
+#endif /* __UNUSED__ */
 
-    for (n = k = 0; k < channels; k++) {
-	if (!gpsdata->used[k])
+    for (n = k = 0; k < gpsdata->satellites_used; k++) {
+	if (gpsdata->used[k] != 0)
 	    continue;
 	satpos[n][0] = sin(gpsdata->azimuth[k]*DEG_2_RAD)
 	    * cos(gpsdata->elevation[k]*DEG_2_RAD);
@@ -329,39 +420,53 @@ void dop(int channels, struct gps_data_t *gpsdata)
 	n++;
     }
 
+#ifdef __UNUSED__
     gpsd_report(0, "Line-of-sight matrix:\n");
     for (k = 0; k < n; k++) {
 	gpsd_report(0, "%f %f %f %f\n",
 		    satpos[k][0], satpos[k][1], satpos[k][2], satpos[k][3]);
     }
+#endif /* __UNUSED__ */
 
-    for (i = 0; i < 4; i++)
-	for (j = 0; j < 4; j++) {
-	    prod[i][j] = 1;
-	    for (k = 0; k < n; k++)
-		prod[i][j] += satpos[k][i] + satpos[k][j];
-	}
+    for (i = 0; i < 4; ++i) { //< rows
+        for (j = 0; j < 4; ++j) { //< cols
+            prod[i][j] = 0.0;
+            for (k = 0; k < n; ++k) {
+                prod[i][j] += satpos[k][i] * satpos[k][j];
+            }
+        }
+    }
 
+#ifdef __UNUSED__
     gpsd_report(0, "product:\n");
     for (k = 0; k < 4; k++) {
 	gpsd_report(0, "%f %f %f %f\n",
 		    prod[k][0], prod[k][1], prod[k][2], prod[k][3]);
     }
+#endif /* __UNUSED__ */
 
     if (invert(prod, inv)) {
+#ifdef __UNUSED__
 	gpsd_report(0, "inverse:\n");
 	for (k = 0; k < 4; k++) {
 	    gpsd_report(0, "%f %f %f %f\n",
 			inv[k][0], inv[k][1], inv[k][2], inv[k][3]);
 	}
-	gpsd_report(0, "HDOP: reported = %f, computed = %f\n",
+	gpsd_report(1, "HDOP: reported = %f, computed = %f\n",
 		    gpsdata->hdop, sqrt(inv[0][0] + inv[1][1]));
-    } else
-	gpsd_report(0, "Matrix is singular.\n");
+#endif /* __UNUSED__ */
+    } else {
+	gpsd_report(1, "LOS matrix is singular, can't calculate DOPs.\n");
+	return 0;
+    }
 
-    //gpsdata->hdop = sqrt(diag[0] + diag[1]);
-    //gpsdata->vdop = sqrt(diag[1]);
-    //gpsdata->pdop = sqrt(diag[0] + diag[1] + diag[2]);
+    /*@ -usedef @*/
+    //gpsdata->hdop = sqrt(inv[0][0] + inv[1][1]);
+    gpsdata->vdop = sqrt(inv[1][1]);
+    gpsdata->pdop = sqrt(inv[0][0] + inv[1][1] + inv[2][2]);
+    gpsdata->tdop = sqrt(inv[3][3]);
+    gpsdata->gdop = sqrt(inv[0][0] + inv[1][1] + inv[2][2] + inv[3][3]);
+    /*@ +usedef @*/
+
+    return VDOP_SET | PDOP_SET | TDOP_SET | GDOP_SET;
 }
-#ifdef __UNUSED__
-#endif /* __UNUSED */
