@@ -8,6 +8,7 @@
 #include <time.h>
 
 #include "gpsd.h"
+#include "timebase.h"
 
 /**************************************************************************
  *
@@ -24,27 +25,27 @@ static void do_lat_lon(char *field[], struct gps_data_t *out)
 
     if (*(p = field[0]) != '\0') {
 	strncpy(str, p, 20);
-	sscanf(p, "%lf", &lat);
+	(void)sscanf(p, "%lf", &lat);
 	m = 100.0 * modf(lat / 100.0, &d);
 	lat = d + m / 60.0;
 	p = field[1];
 	if (*p == 'S')
 	    lat = -lat;
-	if (out->fix.latitude != lat)
-	    out->fix.latitude = lat;
+	if (out->newdata.latitude != lat)
+	    out->newdata.latitude = lat;
 	updated++;
     }
     if (*(p = field[2]) != '\0') {
 	strncpy(str, p, 20);
-	sscanf(p, "%lf", &lon);
+	(void)sscanf(p, "%lf", &lon);
 	m = 100.0 * modf(lon / 100.0, &d);
 	lon = d + m / 60.0;
 
 	p = field[3];
 	if (*p == 'W')
 	    lon = -lon;
-	if (out->fix.longitude != lon)
-	    out->fix.longitude = lon;
+	if (out->newdata.longitude != lon)
+	    out->newdata.longitude = lon;
 	updated++;
     }
 }
@@ -57,25 +58,20 @@ static void do_lat_lon(char *field[], struct gps_data_t *out)
  * Timestamps always look like hhmmss.ss, with the trailing .ss part
  * optional.  RMC has a date field, in the format ddmmyy.  ZDA has
  * separate fields for day/month/year, with a 4-digit year.  This
- * means that for RMC we must supply a century and for GGA and GGL we
+ * means that for RMC we must supply a century and for GGA and GLL we
  * must supply a century, year, and day.  We get the missing data from
- * a previous RMC or ZDA; century in RMC is supplied by the host
- * machine's clock time if there has been no previous RMC.
+ * a previous RMC or ZDA; century in RMC is supplied by a constant if 
+ * there has been no previous RMC.
  *
  **************************************************************************/
 
-#define DD(s)	((s)[0]-'0')*10+((s)[1]-'0')
+#define DD(s)	((int)((s)[0]-'0')*10+(int)((s)[1]-'0'))
 
 static void merge_ddmmyy(char *ddmmyy, struct gps_data_t *out)
 /* sentence supplied ddmmyy, but no century part */
 {
-    if (!out->nmea_date.tm_year) {
-	struct tm tm;
-	time_t now = time(NULL);
-
-	gmtime_r(&now, &tm);
-	out->nmea_date.tm_year = ((1900+tm.tm_year)/100)*100+DD(ddmmyy+4)-1900;
-    }
+    if (out->nmea_date.tm_year == 0)
+	out->nmea_date.tm_year = (CENTURY_BASE + DD(ddmmyy+4)) - 1900;
     out->nmea_date.tm_mon = DD(ddmmyy+2)-1;
     out->nmea_date.tm_mday = DD(ddmmyy);
 }
@@ -101,7 +97,7 @@ static void merge_hhmmss(char *hhmmss, struct gps_data_t *out)
  *
  **************************************************************************/
 
-static int processGPRMC(int count, char *field[], struct gps_data_t *out)
+static gps_mask_t processGPRMC(int count, char *field[], struct gps_data_t *out)
 /* Recommend Minimum Specific GPS/TRANSIT Data */
 {
     /*
@@ -121,42 +117,45 @@ static int processGPRMC(int count, char *field[], struct gps_data_t *out)
 
      * SiRF chipsets don't return either Mode Indicator or magnetic variation.
      */
-    int mask = ERROR_SET;
+    gps_mask_t mask = 0;
 
-    if (!strcmp(field[2], "V")) {
+    if (strcmp(field[2], "V")==0) {
 	/* copes with Magellan EC-10X, see below */
 	if (out->status != STATUS_NO_FIX) {
 	    out->status = STATUS_NO_FIX;
 	    mask |= STATUS_SET;
 	}
-	if (out->fix.mode >= MODE_2D) {
-	    out->fix.mode = MODE_NO_FIX;
+	if (out->newdata.mode >= MODE_2D) {
+	    out->newdata.mode = MODE_NO_FIX;
 	    mask |= MODE_SET;
 	}
-    } else if (!strcmp(field[2], "A")) {
+    } else if (strcmp(field[2], "A")==0) {
 	if (count > 9) {
 	    merge_ddmmyy(field[9], out);
 	    merge_hhmmss(field[1], out);
-	    out->fix.time = out->sentence_time = mkgmtime(&out->nmea_date) + out->subseconds;
+	    mask |= TIME_SET;
+	    out->newdata.time = (double)mkgmtime(&out->nmea_date)+out->subseconds;
+	    if (out->sentence_time != out->newdata.time)
+		mask |= CYCLE_START_SET;
+	    out->sentence_time = out->newdata.time;
 	}
-	mask = TIME_SET;
 	do_lat_lon(&field[3], out);
 	mask |= LATLON_SET;
-	out->fix.speed = atof(field[7]);
-	out->fix.track = atof(field[8]);
+	out->newdata.speed = atof(field[7]) * KNOTS_TO_MPS;
+	out->newdata.track = atof(field[8]);
 	mask |= (TRACK_SET | SPEED_SET);
 	/*
 	 * This copes with GPSes like the Magellan EC-10X that *only* emit
 	 * GPRMC. In this case we set mode and status here so the client
 	 * code that relies on them won't mistakenly believe it has never
-	 * received a fix.
+	 * received a newdata.
 	 */
 	if (out->status == STATUS_NO_FIX) {
 	    out->status = STATUS_FIX;	/* could be DGPS_FIX, we can't tell */
 	    mask |= STATUS_SET;
 	}
-	if (out->fix.mode < MODE_2D) {
-	    out->fix.mode = MODE_2D;
+	if (out->newdata.mode < MODE_2D) {
+	    out->newdata.mode = MODE_2D;
 	    mask |= MODE_SET;
 	}
     }
@@ -164,7 +163,7 @@ static int processGPRMC(int count, char *field[], struct gps_data_t *out)
     return mask;
 }
 
-static int processGPGLL(int count, char *field[], struct gps_data_t *out)
+static gps_mask_t processGPGLL(int count, char *field[], struct gps_data_t *out)
 /* Geographic position - Latitude, Longitude */
 {
     /* Introduced in NMEA 3.0.  Here are the fields:
@@ -191,16 +190,21 @@ static int processGPGLL(int count, char *field[], struct gps_data_t *out)
      * actually ships updates in GPLL that aren't redundant.
      */
     char *status = field[7];
-    int mask = ERROR_SET;
+    gps_mask_t mask = ERROR_SET;
 
-    if (!strcmp(field[6], "A") && (count < 8 || *status != 'N')) {
+    if (strcmp(field[6], "A")==0 && (count < 8 || *status != 'N')) {
 	int newstatus = out->status;
 
 	mask = 0;
 	merge_hhmmss(field[5], out);
-	if (out->nmea_date.tm_year) {
-	    out->fix.time = out->sentence_time = mkgmtime(&out->nmea_date) + out->subseconds;
+	if (out->nmea_date.tm_year != 0) 
+	    gpsd_report(1, "can't use GGA/GGL time until after ZDA or RMC has supplied a year.\n");
+	else {
 	    mask = TIME_SET;
+	    out->newdata.time = (double)mkgmtime(&out->nmea_date)+out->subseconds;
+	    if (out->sentence_time != out->newdata.time)
+		mask |= CYCLE_START_SET;
+	    out->sentence_time = out->newdata.time;
 	}
 	do_lat_lon(&field[1], out);
 	mask |= LATLON_SET;
@@ -216,7 +220,7 @@ static int processGPGLL(int count, char *field[], struct gps_data_t *out)
     return mask;
 }
 
-static int processGPGGA(int c UNUSED, char *field[], struct gps_data_t *out)
+static gps_mask_t processGPGGA(int c UNUSED, char *field[], struct gps_data_t *out)
 /* Global Positioning System Fix Data */
 {
     /*
@@ -233,20 +237,21 @@ static int processGPGGA(int c UNUSED, char *field[], struct gps_data_t *out)
            (empty field) time in seconds since last DGPS update
            (empty field) DGPS station ID number (0000-1023)
     */
-    int mask;
-    double separation;  /* MSL - WGS84) in meters */
+    gps_mask_t mask;
 
     out->status = atoi(field[6]);
     gpsd_report(3, "GPGGA sets status %d\n", out->status);
     mask = STATUS_SET;
     if (out->status > STATUS_NO_FIX) {
 	char *altitude;
-	double oldfixtime = out->fix.time;
+	double oldfixtime = out->newdata.time;
 
 	merge_hhmmss(field[1], out);
-	if (out->nmea_date.tm_year) {
-	    out->fix.time = out->sentence_time = mkgmtime(&out->nmea_date) + out->subseconds;
+	if (out->nmea_date.tm_year == 0) 
+	    gpsd_report(1, "can't use GGA/GGL time until after ZDA or RMC has supplied a year.\n");
+	else {
 	    mask |= TIME_SET;
+	    out->newdata.time = (double)mkgmtime(&out->nmea_date)+out->subseconds;
 	}
 	do_lat_lon(&field[2], out);
 	mask |= LATLON_SET;
@@ -257,15 +262,15 @@ static int processGPGGA(int c UNUSED, char *field[], struct gps_data_t *out)
 	 * See <http://www.sirf.com/Downloads/Technical/apnt0033.pdf>.
 	 * If we see this, force mode to 2D at most.
 	 */
-	if (!altitude[0]) {
-	    if (out->fix.mode == MODE_3D) {
-		out->fix.mode = out->status ? MODE_2D : MODE_NO_FIX; 
+	if (altitude[0] == '\0') {
+	    if (out->newdata.mode == MODE_3D) {
+		out->newdata.mode = out->status ? MODE_2D : MODE_NO_FIX; 
 		mask |= MODE_SET;
 	    }
 	} else {
-	    double oldaltitude = out->fix.altitude;
+	    double oldaltitude = out->newdata.altitude;
 
-	    out->fix.altitude = atof(altitude);
+	    out->newdata.altitude = atof(altitude);
 	    mask |= ALTITUDE_SET;
 
 
@@ -275,28 +280,23 @@ static int processGPGGA(int c UNUSED, char *field[], struct gps_data_t *out)
 	     * SiRF and Garmin chips, which might have some smoothing
 	     * going on.
 	     */
-	    if (oldaltitude == ALTITUDE_NOT_VALID)
-		out->fix.climb = 0;
+	    if (isnan(oldaltitude)==0 || out->newdata.time==oldfixtime)
+		out->newdata.climb = 0;
 	    else {
-		out->fix.climb = (out->fix.altitude-oldaltitude)/(out->fix.time-oldfixtime);
+		out->newdata.climb = (out->newdata.altitude-oldaltitude)/(out->newdata.time-oldfixtime);
 	    }
 	    mask |= CLIMB_SET;
 	}
-	if ( strlen( field[11] ) ) {
-	   separation = atof(field[11] );
+	if (strlen(field[11]) > 0) {
+	   out->separation = atof(field[11]);
 	} else {
-	   separation = ALTITUDE_NOT_VALID;
+	   out->separation = wgs84_separation(out->newdata.latitude,out->newdata.longitude);
 	}
-	gpsd_report(3, "Geoid Separation (MSL - WGS84): from NMEA %lf, calculated %lf\n"
-	    , separation
-	    , wgs84_separation(out->fix.latitude
-		, out->fix.longitude));
-
     }
     return mask;
 }
 
-static int processGPGSA(int c UNUSED, char *field[], struct gps_data_t *out)
+static gps_mask_t processGPGSA(int c UNUSED, char *field[], struct gps_data_t *out)
 /* GPS DOP and Active Satellites */
 {
     /*
@@ -311,30 +311,28 @@ static int processGPGSA(int c UNUSED, char *field[], struct gps_data_t *out)
 	16   = HDOP
 	17   = VDOP
      */
-    int i, mask;
+    gps_mask_t mask;
+    int i;
     
-    out->fix.mode = atoi(field[2]);
+    out->newdata.mode = atoi(field[2]);
     mask = MODE_SET;
-    gpsd_report(3, "GPGSA sets mode %d\n", out->fix.mode);
+    gpsd_report(3, "GPGSA sets mode %d\n", out->newdata.mode);
     out->pdop = atof(field[15]);
     out->hdop = atof(field[16]);
     out->vdop = atof(field[17]);
-    for (i = 0; i < MAXCHANNELS; i++)
-	out->used[i] = 0;
     out->satellites_used = 0;
-    for (i = 0; i < MAXCHANNELS; i++) {
+    memset(out->used,0,sizeof(out->used));
+    for (i = 0; i < NMEA_CHANNELS; i++) {
         int prn = atoi(field[i+3]);
-        if (prn > 0) {
-           out->used[out->satellites_used] = prn;
-           out->satellites_used++;
-       }
+        if (prn > 0)
+	    out->used[out->satellites_used++] = prn;
     }
     mask |= HDOP_SET | VDOP_SET | PDOP_SET;
 
     return mask;
 }
 
-static int processGPGSV(int count, char *field[], struct gps_data_t *out)
+static gps_mask_t processGPGSV(int count, char *field[], struct gps_data_t *out)
 /* GPS Satellites in View */
 {
     /*
@@ -350,15 +348,23 @@ static int processGPGSV(int count, char *field[], struct gps_data_t *out)
                 There my be up to three GSV sentences in a data packet
      */
     int n, fldnum;
-    if (count <= 3)
+    if (count <= 3) {
+	gpsd_zero_satellites(out);
         return ERROR_SET;
+    }
     out->await = atoi(field[1]);
-    if (sscanf(field[2], "%d", &out->part) < 1)
+    if (sscanf(field[2], "%d", &out->part) < 1) {
+	gpsd_zero_satellites(out);
         return ERROR_SET;
-    else if (out->part == 1)
+    } else if (out->part == 1)
 	gpsd_zero_satellites(out);
 
     for (fldnum = 4; fldnum < count; ) {
+	if (out->satellites >= NMEA_CHANNELS) {
+	    gpsd_report(0, "internal error - too many satellites!\n");
+	    gpsd_zero_satellites(out);
+	    break;
+	}
 	out->PRN[out->satellites]       = atoi(field[fldnum++]);
 	out->elevation[out->satellites] = atoi(field[fldnum++]);
 	out->azimuth[out->satellites]   = atoi(field[fldnum++]);
@@ -369,7 +375,7 @@ static int processGPGSV(int count, char *field[], struct gps_data_t *out)
 	 * last sentence in a GPGSV set if the number of satellites is not
 	 * a multiiple of 4.
 	 */
-	if (out->PRN[out->satellites])
+	if (out->PRN[out->satellites] != 0)
 	    out->satellites++;
     }
     if (out->part == out->await && atoi(field[3]) != out->satellites)
@@ -390,16 +396,17 @@ static int processGPGSV(int count, char *field[], struct gps_data_t *out)
      * revision 231.000.000_A2.
      */
     for (n = 0; n < out->satellites; n++)
-	if (out->azimuth[n])
+	if (out->azimuth[n] != 0)
 	    goto sane;
     gpsd_report(3, "Satellite data no good.\n");
+    gpsd_zero_satellites(out);
     return ERROR_SET;
   sane:
     gpsd_report(3, "Satellite data OK.\n");
     return SATELLITE_SET;
     }
 
-static int processPGRME(int c UNUSED, char *field[], struct gps_data_t *out)
+static gps_mask_t processPGRME(int c UNUSED, char *field[], struct gps_data_t *out)
 /* Garmin Estimated Position Error */
 {
     /*
@@ -414,15 +421,16 @@ static int processPGRME(int c UNUSED, char *field[], struct gps_data_t *out)
      * Garmin won't say, but the general belief is that these are 1-sigma.
      * See <http://gpsinformation.net/main/epenew.txt>.
      */
-    out->fix.eph = atof(field[1]);
-    out->fix.epv = atof(field[3]);
+    out->newdata.eph = atof(field[1]);
+    out->newdata.epv = atof(field[3]);
     out->epe = atof(field[5]);
     return HERR_SET | VERR_SET | PERR_SET;
 }
 
-static int processGPZDA(int c UNUSED, char *field[], struct gps_data_t *out)
+static gps_mask_t processGPZDA(int c UNUSED, char *field[], struct gps_data_t *out)
 /* Time & Date */
 {
+    gps_mask_t mask = TIME_SET;
     /*
       $GPZDA,160012.71,11,03,2004,-1,00*7D
       1) UTC time (hours, minutes, seconds, may have fractional subsecond)
@@ -435,10 +443,13 @@ static int processGPZDA(int c UNUSED, char *field[], struct gps_data_t *out)
      */
     merge_hhmmss(field[1], out);
     out->nmea_date.tm_year = atoi(field[4]) - 1900;
-    out->nmea_date.tm_mon = atoi(field[3]);
+    out->nmea_date.tm_mon = atoi(field[3])-1;
     out->nmea_date.tm_mday = atoi(field[2]);
-    out->fix.time = out->sentence_time = mkgmtime(&out->nmea_date) + out->subseconds;
-    return TIME_SET;
+    out->newdata.time = (double)mkgmtime(&out->nmea_date)+out->subseconds;
+    if (out->sentence_time != out->newdata.time)
+	mask |= CYCLE_START_SET;
+    out->sentence_time = out->newdata.time;
+    return mask;
 }
 
 #ifdef __UNUSED__
@@ -452,7 +463,7 @@ static short nmea_checksum(char *sentence, unsigned char *correct_sum)
 	sum ^= c;
     if (correct_sum)
         *correct_sum = sum;
-    snprintf(csum, sizeof(csum), "%02X", sum);
+    (void)snprintf(csum, sizeof(csum), "%02X", sum);
     return(toupper(csum[0])==toupper(p[0]))&&(toupper(csum[1])==toupper(p[1]));
 }
 #endif /* __ UNUSED__ */
@@ -479,13 +490,13 @@ void nmea_add_checksum(char *sentence)
 	p++;
     }
     *p++ = '*';
-    sprintf(p, "%02X\r\n", sum);
+    (void)snprintf(p, 5, "%02X\r\n", (unsigned)sum);
 }
 
-int nmea_parse(char *sentence, struct gps_data_t *outdata)
+gps_mask_t nmea_parse(char *sentence, struct gps_data_t *outdata)
 /* parse an NMEA sentence, unpack it into a session structure */
 {
-    typedef int (*nmea_decoder)(int count, char *f[], struct gps_data_t *out);
+    typedef gps_mask_t (*nmea_decoder)(int count, char *f[], struct gps_data_t *out);
     static struct {
 	char *name;
 	int mask;
@@ -501,7 +512,8 @@ int nmea_parse(char *sentence, struct gps_data_t *outdata)
     };
     unsigned char buf[NMEA_MAX+1];
 
-    int count, retval = 0;
+    int count;
+    gps_mask_t retval = 0;
     unsigned int i;
     char *p, *field[80], *s;
 #ifdef __UNUSED__
@@ -514,22 +526,23 @@ int nmea_parse(char *sentence, struct gps_data_t *outdata)
     }
 #endif /* __ UNUSED__ */
 
+    /*@ -usedef @*//* splint 3.1.1 seems to have a bug here */
     /* make an editable copy of the sentence */
-    strncpy(buf, sentence, NMEA_MAX);
+    strncpy((char *)buf, sentence, NMEA_MAX);
     /* discard the checksum part */
-    for (p = buf; (*p != '*') && (*p >= ' '); ) ++p;
+    for (p = (char *)buf; (*p != '*') && (*p >= ' '); ) ++p;
     *p = '\0';
     /* split sentence copy on commas, filling the field array */
-    for (count = 0, p = buf; p != NULL && *p; ++count, p = strchr(p, ',')) {
-	*p = 0;
+    for (count = 0, p = (char *)buf; p != NULL && *p != '\0'; ++count, p = strchr(p, ',')) {
+	*p = '\0';
 	field[count] = ++p;
     }
     /* dispatch on field zero, the sentence tag */
-    for (i = 0; i < sizeof(nmea_phrase)/sizeof(nmea_phrase[0]); ++i) {
+    for (i = 0; i < (unsigned)(sizeof(nmea_phrase)/sizeof(nmea_phrase[0])); ++i) {
 	s = field[0];
 	if (strlen(nmea_phrase[i].name) == 3)
 	    s += 2;	/* skip talker ID */
-        if (!strcmp(nmea_phrase[i].name, s)) {
+        if (strcmp(nmea_phrase[i].name, s) == 0) {
 	    if (nmea_phrase[i].decoder) {
 		retval = (nmea_phrase[i].decoder)(count, field, outdata);
 		strncpy(outdata->tag, nmea_phrase[i].name, MAXTAGLEN);
@@ -540,23 +553,24 @@ int nmea_parse(char *sentence, struct gps_data_t *outdata)
 	    break;
 	}
     }
+    /*@ +usedef @*/
     return retval;
 }
 
 int nmea_send(int fd, const char *fmt, ... )
 /* ship a command to the GPS, adding * and correct checksum */
 {
-    unsigned int status;
+    int status;
     char buf[BUFSIZ];
     va_list ap;
 
     va_start(ap, fmt) ;
-    vsnprintf(buf, sizeof(buf)-5, fmt, ap);
+    (void)vsnprintf(buf, sizeof(buf)-5, fmt, ap);
     va_end(ap);
     strcat(buf, "*");
     nmea_add_checksum(buf);
-    status = write(fd, buf, strlen(buf));
-    if (status == strlen(buf)) {
+    status = (int)write(fd, buf, strlen(buf));
+    if (status == (int)strlen(buf)) {
 	gpsd_report(2, "=> GPS: %s\n", buf);
 	return status;
     } else {
