@@ -7,8 +7,8 @@ DESCRIPTION:
 
 Initial conditions of the problem:
 
-1. We have a file descriptor open for read. The device on the other end is 
-   sending packets at us.  
+1. We have a file descriptor open for (possibly non-blockin) read. The device 
+   on the other end is sending packets at us.  
 
 2. It may require more than one read to gather a packet.  Reads may span packet
    boundaries.
@@ -31,16 +31,13 @@ distinguish them from baud barf.
 #include <sys/time.h>
 #include <sys/ioctl.h>
 #include "config.h"
-#ifdef HAVE_SYS_FILIO_H
-#include <sys/filio.h>	/* for FIONREAD on BSD systems */
-#endif
-
 #include "gpsd.h"
 
 #ifdef TESTMAIN
 #include <stdarg.h>
 
 static int verbose = 0;
+static int waiting;
 
 void gpsd_report(int errlevel, const char *fmt, ... )
 /* assemble command in printf(3) style, use stderr or syslog */
@@ -51,10 +48,10 @@ void gpsd_report(int errlevel, const char *fmt, ... )
 
 	buf[0] = '\0';
 	va_start(ap, fmt) ;
-	vsnprintf(buf + strlen(buf), sizeof(buf)-strlen(buf), fmt, ap);
+	(void)vsnprintf(buf + strlen(buf), sizeof(buf)-strlen(buf), fmt, ap);
 	va_end(ap);
 
-	fputs(buf, stderr);
+	(void)fputs(buf, stderr);
     }
 }
 #endif /* TESTMAIN */
@@ -62,12 +59,20 @@ void gpsd_report(int errlevel, const char *fmt, ... )
 /* 
  * The packet-recognition state machine.  It doesn't do checksums,
  * caller is responsible for that part.  It can be fooled by garbage
- * that looks like the head of a SiRF packet followed by a NMEA
- * packet; in that case it won't reset until it notices that the SiRF
- * trailer is not where it should be, and the NMEA packet will be
- * lost.  The reverse scenario is not possible because the SiRF leader
- * characters can't occur in an NMEA packet.  Caller should consume a
- * packet when it sees one of the *_RECOGNIZED states.
+ * that looks like the head of a binary packet followed by a NMEA
+ * packet; in that case it won't reset until it notices that the
+ * binary trailer is not where it should be, and the NMEA packet will
+ * be lost.  The reverse scenario is not possible because none of the
+ * binary leader character can occur in an NMEA packet.  Caller should
+ * consume a packet when it sees one of the *_RECOGNIZED states.
+ *
+ * Error handling is brutally simple; any time we see an unexpected
+ * character, go to GROUND_STATE and reset the machine (except that a
+ * $ in an NMEA payload only resets back to NMEA_DOLLAR state).  Because
+ * another good packet will be usually along in less than a second
+ * repeating the same data, Boyer-Moore-like attempts to do parallel
+ * recognition beyond the headers would make no sense in this
+ * application, they'd just add complexity.
  *
  * This state machine allows the following talker IDs:
  *      GP -- Global Positioning System.
@@ -78,7 +83,6 @@ void gpsd_report(int errlevel, const char *fmt, ... )
 enum {
    GROUND_STATE,	/* we don't know what packet type to expect */
    NMEA_DOLLAR,		/* we've seen first character of NMEA leader */
-   SIRF_LEADER_1,	/* we've seen first character of SiRF leader */
 
    NMEA_PUB_LEAD,	/* seen second character of NMEA G leader */
    NMEA_LEADER_END,	/* seen end char of NMEA leader, in body */
@@ -103,6 +107,7 @@ enum {
    EARTHA_5,		/* EARTHA leader H */
 #endif /* EARTHMATE_ENABLE */
 
+   SIRF_LEADER_1,	/* we've seen first character of SiRF leader */
    SIRF_LEADER_2,	/* seen second character of SiRF leader */
    SIRF_ACK_LEAD_1,	/* seen A of possible SiRF Ack */
    SIRF_ACK_LEAD_2,	/* seen c of possible SiRF Ack */
@@ -112,18 +117,28 @@ enum {
    SIRF_TRAILER_1,	/* saw first byte of SiRF trailer */ 
    SIRF_RECOGNIZED,	/* saw second byte of SiRF trailer */
 
+   TSIP_LEADER,		/* we've seen the TSIP leader (DLE) */
+   TSIP_PAYLOAD,	/* we're in TSIP payload */
+   TSIP_DLE,		/* we've seen a DLE in TSIP payload */
+   TSIP_RECOGNIZED,	/* found end of the TSIP packet */
+
    ZODIAC_EXPECTED,	/* expecting Zodiac packet */
    ZODIAC_LEADER_1,	/* saw leading 0xff */
-   ZODIAC_LEADER_2,	/* saw leaing 0x81 */
+   ZODIAC_LEADER_2,	/* saw leading 0x81 */
    ZODIAC_ID_1, 	/* saw first byte of ID */
    ZODIAC_ID_2, 	/* saw second byte of ID */
    ZODIAC_LENGTH_1,	/* saw first byte of Zodiac packet length */
+   ZODIAC_LENGTH_2,	/* saw second byte of Zodiac packet length */
+   ZODIAC_FLAGS_1, 	/* saw first byte of FLAGS */
+   ZODIAC_FLAGS_2, 	/* saw second byte of FLAGS */
+   ZODIAC_HSUM_1, 	/* saw first byte of Header sum */
    ZODIAC_PAYLOAD,	/* we're in a Zodiac payload */
    ZODIAC_RECOGNIZED,	/* found end of the Zodiac packet */
 };
 
 static void nexstate(struct gps_device_t *session, unsigned char c)
 {
+/*@ +charint */
     switch(session->packet_state)
     {
     case GROUND_STATE:
@@ -133,6 +148,10 @@ static void nexstate(struct gps_device_t *session, unsigned char c)
         else if (c == 0xa0)
 	    session->packet_state = SIRF_LEADER_1;
 #endif /* SIRFII_ENABLE */
+#ifdef TSIP_ENABLE
+        else if (c == 0x10)
+	    session->packet_state = TSIP_LEADER;
+#endif /* TSIP_ENABLE */
 #ifdef TRIPMATE_ENABLE
         else if (c == 'A')
 	    session->packet_state = ASTRAL_1;
@@ -141,6 +160,10 @@ static void nexstate(struct gps_device_t *session, unsigned char c)
         else if (c == 'E')
 	    session->packet_state = EARTHA_1;
 #endif /* EARTHMATE_ENABLE */
+#ifdef ZODIAC_ENABLE
+	else if (c == 0xff)
+	    session->packet_state = ZODIAC_LEADER_1;
+#endif /* ZODIAC_ENABLE */
 	break;
     case NMEA_DOLLAR:
 	if (c == 'G')
@@ -166,9 +189,10 @@ static void nexstate(struct gps_device_t *session, unsigned char c)
 	else if (c == '\n')
 	    /* not strictly correct, but helps for interpreting logfiles */
 	    session->packet_state = NMEA_RECOGNIZED;
-	else if (isprint(c))
-	    /* continue gathering body packets */;
-	else
+	else if (c == '$')
+	    /* faster recovery from missing sentence trailers */
+	    session->packet_state = NMEA_DOLLAR;
+	else if (!isprint(c))
 	    session->packet_state = GROUND_STATE;
 	break;
     case NMEA_CR:
@@ -253,13 +277,6 @@ static void nexstate(struct gps_device_t *session, unsigned char c)
 	    session->packet_state = GROUND_STATE;
 	break; 
 #endif /* EARTHMATE_ENABLE */
-#ifdef SIRFII_ENABLE
-    case SIRF_LEADER_1:
-	if (c == 0xa2)
-	    session->packet_state = SIRF_LEADER_2;
-	else
-	    session->packet_state = GROUND_STATE;
-	break;
     case SIRF_ACK_LEAD_1:
 	if (c == 'c')
 	    session->packet_state = SIRF_ACK_LEAD_2;
@@ -272,8 +289,15 @@ static void nexstate(struct gps_device_t *session, unsigned char c)
 	else
 	    session->packet_state = GROUND_STATE;
 	break;
+#ifdef SIRFII_ENABLE
+    case SIRF_LEADER_1:
+	if (c == 0xa2)
+	    session->packet_state = SIRF_LEADER_2;
+	else
+	    session->packet_state = GROUND_STATE;
+	break;
     case SIRF_LEADER_2:
-	session->packet_length = c << 8;
+	session->packet_length = (size_t)(c << 8);
 	session->packet_state = SIRF_LENGTH_1;
 	break;
     case SIRF_LENGTH_1:
@@ -306,8 +330,41 @@ static void nexstate(struct gps_device_t *session, unsigned char c)
 	    session->packet_state = GROUND_STATE;
 	break;
 #endif /* SIRFII_ENABLE */
+#ifdef TSIP_ENABLE
+    case TSIP_LEADER:
+	if (c < 0x13)
+	    session->packet_state = GROUND_STATE;
+	else
+	    session->packet_state = TSIP_PAYLOAD;
+	break;
+    case TSIP_PAYLOAD:
+	if (c == 0x10)
+	    session->packet_state = TSIP_DLE;
+	break;
+    case TSIP_DLE:
+	switch (c)
+	{
+	case 0x03:
+	    session->packet_state = TSIP_RECOGNIZED;
+	    break;
+	case 0x10:
+	    session->packet_state = TSIP_PAYLOAD;
+	    break;
+	default:
+	    session->packet_state = GROUND_STATE;
+	    break;
+	}
+	break;
+    case TSIP_RECOGNIZED:
+        if (c == 0x10)
+	    session->packet_state = TSIP_LEADER;
+	else
+	    session->packet_state = GROUND_STATE;
+	break;
+#endif /* TSIP_ENABLE */
 #ifdef ZODIAC_ENABLE
     case ZODIAC_EXPECTED:
+    case ZODIAC_RECOGNIZED:
 	if (c == 0xff)
 	    session->packet_state = ZODIAC_LEADER_1;
 	else
@@ -326,18 +383,44 @@ static void nexstate(struct gps_device_t *session, unsigned char c)
 	session->packet_state = ZODIAC_ID_2;
 	break;
     case ZODIAC_ID_2:
-	session->packet_length = c;
+	session->packet_length = (size_t)c;
 	session->packet_state = ZODIAC_LENGTH_1;
 	break;
     case ZODIAC_LENGTH_1:
-	/* extra three words are flags, header checkum, and data checksum */
-	session->packet_length += (c << 8);	/* word count! */
-	session->packet_length *= 2;		/* to byte count */
-	if (session->packet_length == 0)
-	    session->packet_length += 4;	/* hdrcheck, flags */
-	else
-	    session->packet_length += 6;	/* hdrcheck, flags, datachk */
-	if (session->packet_length <= MAX_PACKET_LENGTH)
+	session->packet_length += (c << 8);
+	session->packet_state = ZODIAC_LENGTH_2;
+	break;
+    case ZODIAC_LENGTH_2:
+	session->packet_state = ZODIAC_FLAGS_1;
+	break;
+    case ZODIAC_FLAGS_1:
+	session->packet_state = ZODIAC_FLAGS_2;
+	break;
+    case ZODIAC_FLAGS_2:
+	session->packet_state = ZODIAC_HSUM_1;
+	break;
+    case ZODIAC_HSUM_1:
+	{
+ #define getword(i) (short)(session->inbuffer[2*(i)] | (session->inbuffer[2*(i)+1] << 8))
+	    short sum = getword(0) + getword(1) + getword(2) + getword(3);
+	    sum *= -1;
+	    if (sum != getword(4)) {
+		gpsd_report(4, "Zodiac Header checksum 0x%hx expecting 0x%hx\n", 
+		       sum, getword(4));
+		session->packet_state = GROUND_STATE;
+		break;
+	    }
+	}
+	gpsd_report(6,"Zodiac header id=%hd len=%hd flags=%hx\n", getword(1), getword(2), getword(3));
+ #undef getword
+	if (session->packet_length == 0) {
+	    session->packet_state = ZODIAC_RECOGNIZED;
+	    break;
+	}
+	session->packet_length *= 2;		/* word count to byte count */
+	session->packet_length += 2;		/* checksum */
+	/* 10 bytes is the length of the Zodiac header */
+	if (session->packet_length <= MAX_PACKET_LENGTH - 10)
 	    session->packet_state = ZODIAC_PAYLOAD;
 	else
 	    session->packet_state = GROUND_STATE;
@@ -348,188 +431,204 @@ static void nexstate(struct gps_device_t *session, unsigned char c)
 	break;
 #endif /* ZODIAC_ENABLE */
     }
+/*@ -charint */
 }
 
-static void packet_copy(struct gps_device_t *session)
+#ifdef STATE_DEBUG
+static char *buffer_dump(unsigned char *base, unsigned char *end)
+/* dump the state of a specified buffer */
+{
+    static unsigned char buf[BUFSIZ];
+    unsigned char *cp, *tp = buf;
+    for (cp = base; cp < end; cp++)
+	if (isgraph(*cp)) {
+	    *tp++ = *cp;
+	}else {
+	    (void)snprintf(tp, sizeof(buf)-(tp-buf), "\\x%02x", *cp);
+	    tp += 4;
+	}
+    *tp = '\0';
+    return buf;
+}
+#endif /* STATE_DEBUG */
+
+static void packet_accept(struct gps_device_t *session, int packet_type)
 /* packet grab succeeded, move to output buffer */
 {
-    int packetlen = session->inbufptr-session->inbuffer;
-    gpsd_report(6, "Packet copy, type %d\n", session->packet_type);
-    memcpy(session->outbuffer, session->inbuffer, packetlen);
-    session->outbuffer[session->outbuflen = packetlen] = '\0';
+    size_t packetlen = session->inbufptr-session->inbuffer;
+    if (packetlen < sizeof(session->outbuffer)) {
+	memcpy(session->outbuffer, session->inbuffer, packetlen);
+	session->outbuflen = packetlen;
+	session->outbuffer[packetlen] = '\0';
+	session->packet_type = packet_type;
+#ifdef STATE_DEBUG
+	gpsd_report(6, "Packet type %d accepted %d = %s\n", 
+		packet_type, packetlen,
+		buffer_dump(session->outbuffer, 
+			    session->outbuffer+session->outbuflen));
+#endif /* STATE_DEBUG */
+    } else {
+	gpsd_report(1, "Rejected too long packet type %d len %d\n",
+		packet_type,packetlen);
+    }
 }
 
 static void packet_discard(struct gps_device_t *session)
-/* shift the input buffer to discard old data */ 
+/* shift the input buffer to discard all data up to current input pointer */ 
 {
-    int remaining = session->inbuffer + session->inbuflen - session->inbufptr;
-#ifndef TESTMAIN
-    gpsd_report(6, "Packet discard with %d remaining\n", remaining);
-#endif /* TESTMAIN */
-    memmove(session->inbuffer, 
-	    session->inbufptr, 
-	    remaining);
-    session->inbufptr = session->inbuffer;
+    size_t discard = session->inbufptr - session->inbuffer;
+    size_t remaining = session->inbuflen - discard;
+    session->inbufptr = memmove(session->inbuffer, 
+				session->inbufptr, 
+				remaining);
     session->inbuflen = remaining;
+#ifdef STATE_DEBUG
+    gpsd_report(6, "Packet discard of %d, chars remaining is %d = %s\n", 
+		discard, remaining, 
+		buffer_dump(session->inbuffer, 
+			    session->inbuffer + session->inbuflen));
+#endif /* STATE_DEBUG */
+}
+
+static void character_discard(struct gps_device_t *session)
+/* shift the input buffer to discard one character and reread data */
+{
+    memmove(session->inbuffer, session->inbuffer+1, (size_t)--session->inbuflen);
+    session->inbufptr = session->inbuffer;
+#ifdef STATE_DEBUG
+    gpsd_report(6, "Character discarded, buffer %d chars = %s\n",
+		session->inbuflen,
+		buffer_dump(session->inbuffer, 
+			    session->inbuffer + session->inbuflen));
+#endif /* STATE_DEBUG */
 }
 
 /* entry points begin here */
 
-int packet_get(struct gps_device_t *session, int waiting)
+ssize_t packet_get(struct gps_device_t *session)
+/* grab a packet; returns ether BAD_PACKET or the length */
 {
     int newdata;
 #ifndef TESTMAIN
-    int room = sizeof(session->inbuffer)-(session->inbufptr-session->inbuffer);
-
-    if (waiting > room)
-	waiting = room;
-
-    newdata = read(session->gpsdata.gps_fd, session->inbufptr, waiting);
+    /*@ -modobserver @*/
+    newdata = (int)read(session->gpsdata.gps_fd, session->inbufptr, 
+			sizeof(session->inbuffer)-(session->inbufptr-session->inbuffer));
+    /*@ +modobserver @*/
 #else
     newdata = waiting;
 #endif /* TESTMAIN */
 
     if (newdata < 0 && errno != EAGAIN)
 	return BAD_PACKET;
+    else if (newdata == 0 || (newdata < 0 && errno == EAGAIN))
+	return 0;
 
-    {
-	unsigned char buf[BUFSIZ], *cp, *tp = buf;
-	for (cp = session->inbufptr; cp < session->inbufptr + newdata; cp++)
-	    if (isgraph(*cp))
-		*tp++ = *cp;
-	    else {
-		sprintf(tp, "\\x%02x", *cp);
-		tp += 4;
-	    }
-	*tp = '\0';
-	gpsd_report(6, "Read %d chars (total %d): %s\n", newdata, session->inbuflen+newdata, buf);
-    }
-
+#ifdef STATE_DEBUG
+    gpsd_report(6, "Read %d chars to buffer offset %d (total %d): %s\n", 
+		newdata,
+		session->inbufptr-session->inbuffer,
+		session->inbuflen+newdata, 
+		buffer_dump(session->inbufptr, session->inbufptr + newdata));
+#endif /* STATE_DEBUG */
 
     session->outbuflen = 0;
     session->inbuflen += newdata;
     while (session->inbufptr < session->inbuffer + session->inbuflen) {
+	/*@ -modobserver @*/
 	unsigned char c = *session->inbufptr++;
+	/*@ +modobserver @*/
 	nexstate(session, c);
-#ifdef TESTMAINOLD
 	if (isprint(c))
-	    printf("Character '%c', new state: %d\n",c,session->packet_state);
+	    gpsd_report(7, "Character '%c', new state: %d\n",c,session->packet_state);
 	else
-	    printf("Character 0x%02x, new state: %d\n",c,session->packet_state);
-#endif /* TESTMAIN */
+	    gpsd_report(7, "Character 0x%02x, new state: %d\n",c,session->packet_state);
 	if (session->packet_state == GROUND_STATE) {
-#ifdef TESTMAINOLD
-	    gpsd_report(6, "Character discarded\n", session->inbufptr[-1]);
-#endif /* TESTMAIN */
-	    session->inbufptr = memmove(session->inbufptr-1, 
-					session->inbufptr, 
-					session->inbuffer + session->inbuflen - session->inbufptr 
-		);
-	    session->inbuflen--;
+	    character_discard(session);
 	} else if (session->packet_state == NMEA_RECOGNIZED) {
-	    int checksum_ok = 1;
-	    unsigned char csum[3];
-	    unsigned char *trailer = session->inbufptr-5;
+	    bool checksum_ok = true;
+	    char csum[3];
+	    char *trailer = (char *)session->inbufptr-5;
 	    if (*trailer == '*') {
 		unsigned int n, crc = 0;
-		for (n = 1; session->inbuffer + n < trailer; n++)
+		for (n = 1; (char *)session->inbuffer + n < trailer; n++)
 		    crc ^= session->inbuffer[n];
-		snprintf(csum, sizeof(csum), "%02X", crc);
+		(void)snprintf(csum, sizeof(csum), "%02X", crc);
 		checksum_ok = (toupper(csum[0])==toupper(trailer[1])
 				&& toupper(csum[1])==toupper(trailer[2]));
 	    }
-	    if (checksum_ok) {
-		session->packet_type = NMEA_PACKET;
-		packet_copy(session);
-	    } else
-		session->packet_state = GROUND_STATE;
+	    if (checksum_ok)
+		packet_accept(session, NMEA_PACKET);
+	    session->packet_state = GROUND_STATE;
 	    packet_discard(session);
 	} else if (session->packet_state == SIRF_RECOGNIZED) {
 	    unsigned char *trailer = session->inbufptr-4;
-	    unsigned int checksum = (trailer[0] << 8) | trailer[1];
+	    unsigned int checksum = (unsigned)((trailer[0] << 8) | trailer[1]);
 	    unsigned int n, crc = 0;
-	    for (n = 4; n < (size_t)(trailer - session->inbuffer); n++)
-		crc += session->inbuffer[n];
+	    for (n = 4; n < (unsigned)(trailer - session->inbuffer); n++)
+		crc += (int)session->inbuffer[n];
 	    crc &= 0x7fff;
-	    if (checksum == crc) {
-		session->packet_type = SIRF_PACKET;
-		packet_copy(session);
-	    } else
-		session->packet_state = GROUND_STATE;
+	    if (checksum == crc)
+		packet_accept(session, SIRF_PACKET);
+	    session->packet_state = GROUND_STATE;
 	    packet_discard(session);
+#ifdef TSIP_ENABLE
+	} else if (session->packet_state == TSIP_RECOGNIZED) {
+	    if ((session->inbufptr - session->inbuffer) >= 4)
+		packet_accept(session, TSIP_PACKET);
+	    session->packet_state = GROUND_STATE;
+	    packet_discard(session);
+#endif /* TSIP_ENABLE */
 #ifdef ZODIAC_ENABLE
 	} else if (session->packet_state == ZODIAC_RECOGNIZED) {
- #define getw(i) (session->inbuffer[2*(i)] | (session->inbuffer[2*(i)+1] << 8))
-	    short len, n, crc1, crc2;
-	    for (n = crc1 = 0; n < 4; n++)
-		crc1 += (short)getw(n);
-	    crc1 *= -1;
-	    gpsd_report(6, "Header checksum 0x%hx expecting 0x%hx\n", 
-		   crc1, (short)getw(4));
-	    len = getw(2);
-	    for (n = crc2 = 0; n < len; n++)
-		crc2 += (short)getw(5+n);
-	    crc2 *= -1;
-	    gpsd_report(6,
-		"Data checksum 0x%hx over length %d, expecting 0x%hx\n", 
-			crc2, len, getw(5 + len));
-	    if (crc2 == (short)getw(5 + len) && crc1 == (short)getw(4)) {
-		session->packet_type = ZODIAC_PACKET;
-		packet_copy(session);
-	    } else
-		session->packet_state = GROUND_STATE;
+ #define getword(i) (short)(session->inbuffer[2*(i)] | (session->inbuffer[2*(i)+1] << 8))
+	    short len, n, sum;
+	    len = getword(2);
+	    for (n = sum = 0; n < len; n++)
+		sum += getword(5+n);
+	    sum *= -1;
+	    if (len == 0 || sum == getword(5 + len)) {
+		packet_accept(session, ZODIAC_PACKET);
+	    } else {
+		gpsd_report(4,
+		    "Zodiac Data checksum 0x%hx over length %hd, expecting 0x%hx\n", 
+			sum, len, getword(5 + len));
+	    }
+	    session->packet_state = GROUND_STATE;
 	    packet_discard(session);
-#undef getw
+#undef getword
 #endif /* ZODIAC_ENABLE */
 	}
     }
 
-    return session->outbuflen;
+    return (ssize_t)session->outbuflen;
 }
 
-/*
- * This constant controls how long the packet sniffer will spend looking
- * for a packet leader before it gives up.  It *must* be larger than
- * MAX_PACKET_LENGTH or we risk never syncing up at all.  Large values
- * will produce annoying startup lag.
- */
-#define SNIFF_RETRIES	600
-
-int packet_sniff(struct gps_device_t *session)
-/* try to sync up with the packet stream */
+void packet_reset(struct gps_device_t *session)
+/* return the packet machine to the ground state */
 {
-    unsigned int n, count = 0;
     session->packet_type = BAD_PACKET;
     session->packet_state = GROUND_STATE;
     session->inbuflen = 0;
     session->inbufptr = session->inbuffer;
-
-    gpsd_report(5, "packet_sniff begins\n");
-    for (n = 0; n < SNIFF_RETRIES; n += count) {
-	count = 0;
-	if (ioctl(session->gpsdata.gps_fd, FIONREAD, &count) < 0)
-	    return BAD_PACKET;
-	if (count && packet_get(session, count)) {
-	    /* push back the last packet grabbed */
-	    if (session->outbuflen + session->inbuflen < MAX_PACKET_LENGTH) {
-		memmove(session->inbuffer+session->outbuflen,
-			session->inbuffer,
-			session->inbuflen);
-		memmove(session->inbuffer,
-			session->outbuffer,
-			session->outbuflen);
-		session->inbuflen += session->outbuflen;
-		session->outbuflen = 0;
-	    }
-	    gpsd_report(5, "packet_sniff ends\n");
-	    return session->packet_type;
-	}
-    }
-
-    gpsd_report(5, "packet_sniff found no packet\n");
-    return BAD_PACKET;
 }
 
+#ifdef __UNUSED__
+void packet_pushback(struct gps_device_t *session)
+/* push back the last packet grabbed */
+{
+    if (session->outbuflen + session->inbuflen < MAX_PACKET_LENGTH) {
+	memmove(session->inbuffer+session->outbuflen,
+		session->inbuffer,
+		session->inbuflen);
+	memmove(session->inbuffer,
+		session->outbuffer,
+		session->outbuflen);
+	session->inbuflen += session->outbuflen;
+	session->outbuflen = 0;
+    }
+}
+#endif /* __UNUSED */
+
 #ifdef TESTMAIN
 /* To build a test main, compile with cc -DTESTMAIN -g packet.c -o packet */
 
@@ -675,7 +774,8 @@ int main(int argc, char *argv[])
 	state.inbuflen = 0;
 	memcpy(state.inbufptr = state.inbuffer, mp->test, mp->testlen);
 	gpsd_report(2, "%s starts with state %d\n", mp->legend, mp->initstate);
-	st = packet_get(&state, mp->testlen);
+	waiting = mp->testlen;
+	st = packet_get(&state);
 	if (state.packet_type != mp->type)
 	    printf("%s test FAILED (packet type %d wrong).\n", mp->legend, st);
 	else if (memcmp(mp->test + mp->garbage_offset, state.outbuffer, state.outbuflen))
@@ -687,17 +787,17 @@ int main(int argc, char *argv[])
 	     cp < state.outbuffer + state.outbuflen; 
 	     cp++) {
 	    if (st != NMEA_PACKET)
-		printf(" 0x%02x", *cp);
+		(void)printf(" 0x%02x", *cp);
 	    else if (*cp == '\r')
-		fputs("\\r", stdout);
+		(void)fputs("\\r", stdout);
 	    else if (*cp == '\n')
-		fputs("\\n", stdout);
+		(void)fputs("\\n", stdout);
 	    else if (isprint(*cp))
-		putchar(*cp);
+		(void)putchar(*cp);
 	    else
-		printf("\\x%02x", *cp);
+		(void)printf("\\x%02x", *cp);
 	}
-	putchar('\n');
+	(void)putchar('\n');
 #endif /* DUMPIT */
     }
 }
