@@ -18,25 +18,6 @@
  * accelerometer inputs to the chip to allow it to continue 
  * to navigate in the absence of satellite information, and 
  * to improve fixes when you do have satellites.)"
- *
- * Why we only use packet 42 (Geodetic Navigation Information) conditionally: 
- *
- * Many versions of the SiRF protocol manual don't document 
- * this sentence at all.  Those that do may incorrectly
- * describe UTC Day, Hour, and Minute as 2-byte quantities,
- * not 1-byte. Chris Kuethe, our SiRF expert, tells us:
- *
- * "The Geodetic Navigation packet (0x29) was not fully
- * implemented in firmware prior to version 2.3.2. So for
- * anyone running 231.000.000 or earlier (including ES,
- * SiRFDRive, XTrac trains) you won't get UTC time. I don't
- * know what's broken in firmwares before 2.3.1..."
- *
- * To work around the incomplete implementation of this
- * packet in 231, we used to assume that only the altitude field
- * from this packet is valid.  But even this doesn't necessarily
- * seem to be the case.  Instead, we do our own computation 
- * of geoid separation now.
  */
 
 #include <stdio.h>
@@ -50,40 +31,45 @@
 #include <stdio.h>
 
 #include "gpsd.h"
+#include "bits.h"
 #if defined(SIRFII_ENABLE) && defined(BINARY_ENABLE)
 
 #define HI(n)		((n) >> 8)
 #define LO(n)		((n) & 0xff)
 
-static u_int16_t sirf_write(int fd, u_int8_t *msg) {
-   int       i, len, ok, crc;
+static bool sirf_write(int fd, unsigned char *msg) {
+   unsigned int       crc;
+   size_t    i, len;
    char	     buf[MAX_PACKET_LENGTH*2];
+   bool      ok;
 
-   len = (msg[2] << 8) | msg[3];
+   len = (size_t)((msg[2] << 8) | msg[3]);
 
    /* calculate CRC */
    crc = 0;
    for (i = 0; i < len; i++)
-	crc += msg[4 + i];
+	crc += (int)msg[4 + i];
    crc &= 0x7fff;
 
    /* enter CRC after payload */
-   msg[len + 4] = (u_int8_t)((crc & 0xff00) >> 8);
-   msg[len + 5] = (u_int8_t)( crc & 0x00ff);
+   msg[len + 4] = (unsigned char)((crc & 0xff00) >> 8);
+   msg[len + 5] = (unsigned char)( crc & 0x00ff);
 
    buf[0] = '\0';
    for (i = 0; i < len+8; i++)
-       sprintf(buf+strlen(buf), " %02x", msg[i]);
+       (void)snprintf(buf+strlen(buf),sizeof(buf)-strlen(buf),
+		      " %02x", (unsigned)msg[i]);
    gpsd_report(4, "Writing SiRF control type %02x:%s\n", msg[4], buf);
-   ok = write(fd, msg, len+8) == len+8;
-   tcdrain(fd);
+   ok = (write(fd, msg, len+8) == (ssize_t)(len+8));
+   (void)tcdrain(fd);
    return(ok);
 }
 
-static int sirf_speed(int ttyfd, int speed) 
+static bool sirf_speed(int ttyfd, speed_t speed) 
 /* change speed in binary mode */
 {
-   u_int8_t msg[] = {0xa0, 0xa2, 0x00, 0x09,
+    /*@ +charint @*/
+   unsigned char msg[] = {0xa0, 0xa2, 0x00, 0x09,
                      0x86, 
                      0x0, 0x0, 0x12, 0xc0,	/* 4800 bps */
 		     0x08,			/* 8 data bits */
@@ -91,16 +77,18 @@ static int sirf_speed(int ttyfd, int speed)
 		     0x00,			/* no parity */
 		     0x00,			/* reserved */
                      0x00, 0x00, 0xb0, 0xb3};
+   /*@ -charint @*/
 
-   msg[7] = HI(speed);
-   msg[8] = LO(speed);
+   msg[7] = (unsigned char)HI(speed);
+   msg[8] = (unsigned char)LO(speed);
    return (sirf_write(ttyfd, msg));
 }
 
-static int sirf_to_nmea(int ttyfd, int speed) 
+static bool sirf_to_nmea(int ttyfd, speed_t speed) 
 /* switch from binary to NMEA at specified baud */
 {
-   u_int8_t msg[] = {0xa0, 0xa2, 0x00, 0x18,
+    /*@ +charint @*/
+   unsigned char msg[] = {0xa0, 0xa2, 0x00, 0x18,
                      0x81, 0x02,
                      0x01, 0x01, /* GGA */
                      0x00, 0x00, /* suppress GLL */
@@ -112,31 +100,40 @@ static int sirf_to_nmea(int ttyfd, int speed)
                      0x00, 0x01, 0x00, 0x01,
                      0x12, 0xc0, /* 4800 bps */
                      0x00, 0x00, 0xb0, 0xb3};
+   /*@ -charint @*/
 
-   msg[26] = HI(speed);
-   msg[27] = LO(speed);
+   msg[26] = (unsigned char)HI(speed);
+   msg[27] = (unsigned char)LO(speed);
    return (sirf_write(ttyfd, msg));
 }
 
-#define getb(off)	(buf[off])
-#define getw(off)	((short)((getb(off) << 8) | getb(off+1)))
-#define getl(off)	((int)((getw(off) << 16) | (getw(off+2) & 0xffff)))
-
 static void sirfbin_mode(struct gps_device_t *session, int mode)
 {
     if (mode == 0) {
-	gpsd_switch_driver(session, "SiRF-II NMEA");
-	sirf_to_nmea(session->gpsdata.gps_fd,session->gpsdata.baudrate);
+	(void)gpsd_switch_driver(session, "SiRF-II NMEA");
+	(void)sirf_to_nmea(session->gpsdata.gps_fd,session->gpsdata.baudrate);
 	session->gpsdata.driver_mode = 0;
     }
 }
 
-int sirf_parse(struct gps_device_t *session, unsigned char *buf, int len)
+gps_mask_t sirf_parse(struct gps_device_t *session, unsigned char *buf, size_t len)
 {
-    int	st, i, j, cn, navtype, mask;
-    char buf2[MAX_PACKET_LENGTH*3] = "";
+    int	st, i, j, cn;
+    unsigned short navtype;
+    gps_mask_t mask;
+    char buf2[MAX_PACKET_LENGTH*3+2];
     double fv;
-    u_int8_t disablesubframe[] = {0xa0, 0xa2, 0x00, 0x19,
+    /*@ +charint @*/
+    unsigned char enablesubframe[] = {0xa0, 0xa2, 0x00, 0x19,
+				 0x80, 0x00, 0x00, 0x00,
+				 0x00, 0x00, 0x00, 0x00,
+				 0x00, 0x00, 0x00, 0x00,
+				 0x00, 0x00, 0x00, 0x00,
+				 0x00, 0x00, 0x00, 0x00,
+				 0x00, 0x00, 0x00, 0x0C,
+				 0x10,
+				 0x00, 0x00, 0xb0, 0xb3};
+    unsigned char disablesubframe[] = {0xa0, 0xa2, 0x00, 0x19,
 				  0x80, 0x00, 0x00, 0x00,
 				  0x00, 0x00, 0x00, 0x00,
 				  0x00, 0x00, 0x00, 0x00,
@@ -146,128 +143,154 @@ int sirf_parse(struct gps_device_t *session, unsigned char *buf, int len)
 				  0x00,
 				  0x00, 0x00, 0xb0, 0xb3};
 
+    /*@ -charint @*/
+    if (len == 0)
+	return 0;
+
     buf2[0] = '\0';
-    for (i = 0; i < len; i++)
-	sprintf(buf2+strlen(buf2), "%02x", buf[i]);
-    gpsd_report(5, "Raw SiRF packet type %d length %d: %s\n", buf[0],len,buf2);
-    snprintf(session->gpsdata.tag,sizeof(session->gpsdata.tag),"MID%d",buf[0]);
+    for (i = 0; i < (int)len; i++)
+	(void)snprintf(buf2+strlen(buf2), 
+		       sizeof(buf2)-strlen(buf2),
+		       "%02x", (unsigned int)buf[i]);
+    strcat(buf2, "\n");
+    buf += 4;
+    len -= 8;
+    gpsd_report(5, "Raw SiRF packet type 0x%02x length %d: %s\n", buf[0],len,buf2);
+    (void)snprintf(session->gpsdata.tag, sizeof(session->gpsdata.tag),
+		   "MID%d",(int)buf[0]);
 
     switch (buf[0])
     {
     case 0x02:		/* Measure Navigation Data Out */
 	mask = 0;
-	if (!(session->driverstate & (SIRF_GE_232 | UBLOX))) {
+	session->gpsdata.satellites_used = (int)getub(buf, 28);
+	memset(session->gpsdata.used,0,sizeof(session->gpsdata.used));
+	for (i = 0; i < SIRF_CHANNELS; i++)
+	    session->gpsdata.used[i] = (int)getub(buf, 29+i);
+	if ((session->sirf.driverstate & (SIRF_GE_232 | UBLOX))==0) {
 	    /* position/velocity is bytes 1-18 */
-	    ecef_to_wgs84fix(&session->gpsdata.fix, 
-			     getl(1), getl(5), getl(9),
-			     getw(13)/8.0, getw(15)/8.0, getw(17)/8.0);
+	    ecef_to_wgs84fix(&session->gpsdata, 
+			     getsl(buf, 1)*1.0, getsl(buf, 5)*1.0, getsl(buf, 9)*1.0,
+			     getsw(buf, 13)/8.0, getsw(buf, 15)/8.0, getsw(buf, 17)/8.0);
 	    /* WGS 84 geodesy parameters */
 	    /* fix status is byte 19 */
-	    navtype = getb(19);
+	    navtype = (unsigned short)getub(buf, 19);
 	    session->gpsdata.status = STATUS_NO_FIX;
-	    session->gpsdata.fix.mode = MODE_NO_FIX;
-	    if (navtype & 0x80)
+	    session->gpsdata.newdata.mode = MODE_NO_FIX;
+	    if ((navtype & 0x80) != 0)
 		session->gpsdata.status = STATUS_DGPS_FIX;
 	    else if ((navtype & 0x07) > 0 && (navtype & 0x07) < 7)
 		session->gpsdata.status = STATUS_FIX;
 	    if ((navtype & 0x07) == 4 || (navtype & 0x07) == 6)
-		session->gpsdata.fix.mode = MODE_3D;
-	    else if (session->gpsdata.status)
-		session->gpsdata.fix.mode = MODE_2D;
-	    if (session->gpsdata.fix.mode == MODE_3D)
-		mask |= ALTITUDE_SET;
+		session->gpsdata.newdata.mode = MODE_3D;
+	    else if (session->gpsdata.status != 0)
+		session->gpsdata.newdata.mode = MODE_2D;
+	    if (session->gpsdata.newdata.mode == MODE_3D)
+		mask |= ALTITUDE_SET | CLIMB_SET;
 	    gpsd_report(4, "MND 0x02: Navtype = 0x%0x, Status = %d, mode = %d\n", 
-			navtype,session->gpsdata.status,session->gpsdata.fix.mode);
+			navtype,session->gpsdata.status,session->gpsdata.newdata.mode);
 	    /* byte 20 is HDOP, see below */
 	    /* byte 21 is "mode 2", not clear how to interpret that */ 
-	    session->gpsdata.fix.time = session->gpsdata.sentence_time
-		= gpstime_to_unix(getw(22), getl(24)*1e-2, -session->context->leap_seconds);
+	    session->gpsdata.newdata.time = session->gpsdata.sentence_time
+		= gpstime_to_unix(getsw(buf, 22), getul(buf, 24)*1e-2) - session->context->leap_seconds;
 #ifdef NTPSHM_ENABLE
-	    session->time_seen |= TIME_SEEN_GPS_2;
-	    if (IS_HIGHEST_BIT(session->time_seen,TIME_SEEN_GPS_2))
-		ntpshm_put(session, session->gpsdata.fix.time);
+	    if (session->gpsdata.newdata.mode > MODE_NO_FIX) {
+		if ((session->sirf.time_seen & TIME_SEEN_GPS_2) == 0)
+		    gpsd_report(4, "valid time in message 0x02, seen=0x%02x\n",
+				session->sirf.time_seen);
+		session->sirf.time_seen |= TIME_SEEN_GPS_2;
+		if (IS_HIGHEST_BIT(session->sirf.time_seen,TIME_SEEN_GPS_2))
+		    (void)ntpshm_put(session, session->gpsdata.newdata.time + 0.8);
+	    }
 #endif /* NTPSHM_ENABLE */
-
-	    gpsd_binary_fix_dump(session, buf2);
 	    /* fix quality data */
-	    session->gpsdata.hdop = getb(20)/5.0;
-	    session->gpsdata.satellites_used = getb(28);
-#ifdef __UNUSED__
-	    if (session->gpsdata.satellites)
-		dop(session->gpsdata.satellites_used, &session->gpsdata);
-#endif /* __UNUSED */
-	    for (i = 0; i < MAXCHANNELS; i++)
-		session->gpsdata.used[i] = getb(29+i);
-	    session->gpsdata.pdop = session->gpsdata.vdop = 0.0;
-	    gpsd_binary_quality_dump(session, buf2 + strlen(buf2));
-	    gpsd_report(3, "<= GPS: %s", buf2);
-	    mask |= TIME_SET | LATLON_SET | TRACK_SET | SPEED_SET | STATUS_SET | MODE_SET | HDOP_SET;
+	    session->gpsdata.hdop = (double)getub(buf, 20)/5.0;
+	    mask |= TIME_SET | LATLON_SET | TRACK_SET | SPEED_SET | STATUS_SET | MODE_SET | HDOP_SET | CYCLE_START_SET;
 	}
 	return mask;
 
     case 0x04:		/* Measured tracker data out */
 	gpsd_zero_satellites(&session->gpsdata);
-	for (i = st = 0; i < MAXCHANNELS; i++) {
-	    int good, off = 8 + 15 * i;
-	    session->gpsdata.PRN[st]       = getb(off);
-	    session->gpsdata.azimuth[st]   = (int)((getb(off+1)*3)/2.0);
-	    session->gpsdata.elevation[st] = (int)(getb(off+2)/2.0);
+	session->gpsdata.sentence_time
+	    = gpstime_to_unix(getsw(buf, 1), getul(buf, 3)*1e-2) - session->context->leap_seconds;
+	for (i = st = 0; i < SIRF_CHANNELS; i++) {
+	    int off = 8 + 15 * i;
+	    bool good;
+	    session->gpsdata.PRN[st]       = (int)getub(buf, off);
+	    session->gpsdata.azimuth[st]   = (int)(((unsigned)getub(buf, off+1)*3)/2.0);
+	    session->gpsdata.elevation[st] = (int)((unsigned)getub(buf, off+2)/2.0);
 	    cn = 0;
 	    for (j = 0; j < 10; j++)
-		cn += getb(off+5+j);
+		cn += (int)getub(buf, off+5+j);
 	    session->gpsdata.ss[st] = cn/10;
-	    // session->gpsdata.used[st] = (getw(off+3) == 0xbf);
-	    good = session->gpsdata.PRN[st] && 
-		session->gpsdata.azimuth[st] && 
-		session->gpsdata.elevation[st];
-	    session->gpsdata.sentence_time
-		= gpstime_to_unix(getw(1), getl(3)*1e-2, -session->context->leap_seconds);
-#ifdef NTPSHM_ENABLE
-	    session->time_seen |= TIME_SEEN_GPS_1;
-	    if (IS_HIGHEST_BIT(session->time_seen,TIME_SEEN_GPS_1))
-		ntpshm_put(session, session->gpsdata.sentence_time);
-#endif /* NTPSHM_ENABLE */
+	    good = session->gpsdata.PRN[st]!=0 && 
+		session->gpsdata.azimuth[st]!=0 && 
+		session->gpsdata.elevation[st]!=0;
 #ifdef __UNUSED__
 	    gpsd_report(4, "PRN=%2d El=%3.2f Az=%3.2f ss=%3d stat=%04x %c\n",
-			getb(off), 
-			getb(off+2)/2.0, 
-			(getb(off+1)*3)/2.0,
+			getub(buf, off), 
+			getub(buf, off+2)/2.0, 
+			(getub(buf, off+1)*3)/2.0,
 			cn/10, 
-			getw(off+3),
+			getuw(buf, off+3),
 			good ? '*' : ' ');
 #endif /* UNUSED */
-	    if (good)
+	    if (good!=0)
 		st += 1;
 	}
 	session->gpsdata.satellites = st;
+#ifdef NTPSHM_ENABLE
+	if (st > 3) {
+	    if ((session->sirf.time_seen & TIME_SEEN_GPS_1)==0)
+		gpsd_report(4, "valid time in message 0x04, seen=0x%02x\n",
+			    session->sirf.time_seen);
+	    session->sirf.time_seen |= TIME_SEEN_GPS_1;
+	    if (IS_HIGHEST_BIT(session->sirf.time_seen,TIME_SEEN_GPS_1))
+		(void)ntpshm_put(session,session->gpsdata.sentence_time+0.8);
+	}
+#endif /* NTPSHM_ENABLE */
 	/*
 	 * The freaking brain-dead SiRF chip doesn't obey its own
 	 * rate-control command for 04, at least at firmware rev. 231, 
 	 * so we have to do our own rate-limiting here...
 	 */
-	if (session->counter % 5)
+	if ((session->sirf.satcounter++ % 5) != 0)
 	    break;
-	gpsd_binary_satellite_dump(session, buf2);
 	gpsd_report(4, "MTD 0x04: %d satellites\n", st);
-	gpsd_report(3, "<= GPS: %s", buf2);
 	return TIME_SET | SATELLITE_SET;
 
+    case 0x05:		/* Raw Tracker Data Out */
+	return 0;
+
     case 0x06:		/* Software Version String */
-	gpsd_report(4, "FV  0x06: Firmware version: %s\n", 
-		    session->outbuffer+5);
-	fv = atof(session->outbuffer+5);
+	gpsd_report(4, "FV  0x06: Firmware version: %s\n", buf+1);
+	fv = atof((char *)(buf+1));
 	if (fv < 231) {
-	    session->driverstate |= SIRF_LT_231;
-	    sirfbin_mode(session, 0);
+	    session->sirf.driverstate |= SIRF_LT_231;
+	    if (fv > 200)
+		sirfbin_mode(session, 0);
 	} else if (fv < 232) 
-	    session->driverstate |= SIRF_EQ_231;
+	    session->sirf.driverstate |= SIRF_EQ_231;
 	else {
-	    session->driverstate |= SIRF_GE_232;
+	    /*@ +charint @*/
+	    unsigned char enablemid52[] = {
+		0xa0, 0xa2, 0x00, 0x08, 
+		0xa6, 0x00, 0x34, 0x01, 0x00, 0x00, 0x00, 0x00,
+		0x00, 0xdb, 0xb0, 0xb3};
+	    /*@ -charint @*/
+	    gpsd_report(4, "Enabling PPS message...\n");
+	    (void)sirf_write(session->gpsdata.gps_fd, enablemid52);
+	    session->sirf.driverstate |= SIRF_GE_232;
 	    session->context->valid |= LEAP_SECOND_VALID;
 	}
-	if (strstr(session->outbuffer+5, "ES"))
+	if (strstr((char *)(buf+1), "ES"))
 	    gpsd_report(4, "Firmware has XTrac capability\n");
-	gpsd_report(4, "Driver state flags are: %0x\n", session->driverstate);
+	gpsd_report(4, "Driver state flags are: %0x\n", session->sirf.driverstate);
+	session->sirf.time_seen = 0;
+	if ((session->context->valid & LEAP_SECOND_VALID)==0) {
+	    gpsd_report(4, "Enabling subframe transmission...\n");
+	    (void)sirf_write(session->gpsdata.gps_fd, enablesubframe);
+	}
 	return 0;
 
     case 0x08:
@@ -289,20 +312,20 @@ int sirf_parse(struct gps_device_t *session, unsigned char *buf, int len)
 	 * changes 1 second every few years. Maybe."
 	 */
         {
-	    unsigned int i, pageid, subframe, leap, words[10];
-	    unsigned int svid = getb(1);
-	    unsigned int chan = getb(2);
-	    words[0] = getl(3);
-	    words[1] = getl(7);
-	    words[2] = getl(11);
-	    words[3] = getl(15);
-	    words[4] = getl(19);
-	    words[5] = getl(23);
-	    words[6] = getl(27);
-	    words[7] = getl(31);
-	    words[8] = getl(35);
-	    words[9] = getl(39);
-	    gpsd_report(5, "50B (raw): CH=%d, SV=%d %08x %08x %08x %08x %08x %08x %08x %08x %08x %08x\n", 
+	    unsigned int pageid, subframe, leap, words[10];
+	    unsigned int chan = (unsigned int)getub(buf, 1);
+	    unsigned int svid = (unsigned int)getub(buf, 2);
+	    words[0] = (unsigned int)getul(buf, 3);
+	    words[1] = (unsigned int)getul(buf, 7);
+	    words[2] = (unsigned int)getul(buf, 11);
+	    words[3] = (unsigned int)getul(buf, 15);
+	    words[4] = (unsigned int)getul(buf, 19);
+	    words[5] = (unsigned int)getul(buf, 23);
+	    words[6] = (unsigned int)getul(buf, 27);
+	    words[7] = (unsigned int)getul(buf, 31);
+	    words[8] = (unsigned int)getul(buf, 35);
+	    words[9] = (unsigned int)getul(buf, 39);
+	    gpsd_report(4, "50B (raw): CH=%d, SV=%d %08x %08x %08x %08x %08x %08x %08x %08x %08x %08x\n", 
 	    	chan, svid, 
 	    	words[0], words[1], words[2], words[3], words[4], 
 	    	words[5], words[6], words[7], words[8], words[9]);
@@ -340,10 +363,10 @@ int sirf_parse(struct gps_device_t *session, unsigned char *buf, int len)
 	     * See page 105 for the mapping between magic SVIDs and pages.
 	     */
 	    pageid = (words[2] & 0x3F0000) >> 16;
-	    gpsd_report(5, "Subframe 4 SVID is %d\n", pageid);
+	    gpsd_report(2, "Subframe 4 SVID is %d\n", pageid);
 	    if (pageid == 56) {	/* magic SVID for page 18 */
 		/* once we've filtered, we can ignore the TEL and HOW words */
-		gpsd_report(5, "50B: CH=%d, SV=%d SF=%d %06x %06x %06x %06x %06x %06x %06x %06x\n", 
+		gpsd_report(2, "50B: CH=%d, SV=%d SF=%d %06x %06x %06x %06x %06x %06x %06x %06x\n", 
 			    chan, svid, subframe,
 				words[2], words[3], words[4], words[5], 
 				words[6], words[7], words[8], words[9]);
@@ -378,50 +401,53 @@ int sirf_parse(struct gps_device_t *session, unsigned char *buf, int len)
 		if (leap > 128)
 		    leap ^= 0xff;
 		gpsd_report(2, "leap-seconds is %d\n", leap);
-		session->context->leap_seconds = leap;
-		session->context->valid = LEAP_SECOND_VALID;
+		session->context->leap_seconds = (int)leap;
+		session->context->valid |= LEAP_SECOND_VALID;
+	    }
+
+	    if (session->context->valid & LEAP_SECOND_VALID) {
 		gpsd_report(4, "Disabling subframe transmission...\n");
-		sirf_write(session->gpsdata.gps_fd, disablesubframe);
+		(void)sirf_write(session->gpsdata.gps_fd, disablesubframe);
 	    }
 	}
 	break;
     case 0x09:		/* CPU Throughput */
 	gpsd_report(4, 
 		    "THR 0x09: SegStatMax=%.3f, SegStatLat=%3.f, AveTrkTime=%.3f, Last MS=%3.f\n", 
-		    (float)getw(1)/186, (float)getw(3)/186, 
-		    (float)getw(5)/186, getw(7));
+		    (float)getuw(buf, 1)/186, (float)getuw(buf, 3)/186, 
+		    (float)getuw(buf, 5)/186, getuw(buf, 7));
     	return 0;
 
     case 0x0a:		/* Error ID Data */
-	switch (getw(1))
+	switch (getuw(buf, 1))
 	{
 	case 2:
-	    gpsd_report(4, "EID 0x0a type 2: Subframe error on PRN %ld\n", getl(5));
+	    gpsd_report(4, "EID 0x0a type 2: Subframe %d error on PRN %ld\n", getul(buf, 9), getul(buf, 5));
 	    break;
 
 	case 4107:
-	    gpsd_report(4, "EID 0x0a type 4177: neither KF nor LSQ fix.\n", getl(5));
+	    gpsd_report(4, "EID 0x0a type 4107: neither KF nor LSQ fix.\n", getul(buf, 5));
 	    break;
 
 	default:
-	    gpsd_report(4, "EID 0x0a: Error ID type %d\n", getw(1));
+	    gpsd_report(4, "EID 0x0a: Error ID type %d\n", getuw(buf, 1));
 	    break;
 	}
 	return 0;
 
     case 0x0b:		/* Command Acknowledgement */
-	gpsd_report(4, "ACK 0x0b: %02x\n",getb(1));
+	gpsd_report(4, "ACK 0x0b: %02x\n",getub(buf, 1));
     	return 0;
 
     case 0x0c:		/* Command NAcknowledgement */
-	gpsd_report(4, "NAK 0x0c: %02x\n",getb(1));
+	gpsd_report(4, "NAK 0x0c: %02x\n",getub(buf, 1));
     	return 0;
 
     case 0x0d:		/* Visible List */
 	return 0;
 
     case 0x12:		/* OK To Send */
-	gpsd_report(4, "OTS 0x12: send indicator = %d\n",getb(1));
+	gpsd_report(4, "OTS 0x12: send indicator = %d\n",getub(buf, 1));
 	return 0;
 
     case 0x1b:		/* DGPS status (undocumented) */
@@ -429,7 +455,7 @@ int sirf_parse(struct gps_device_t *session, unsigned char *buf, int len)
 
     case 0x29:		/* Geodetic Navigation Information */
 	mask = 0;
-	if (session->driverstate & SIRF_GE_232) {
+	if (session->sirf.driverstate & SIRF_GE_232) {
 	    /*
 	     * Many versions of the SiRF protocol manual don't document 
 	     * this sentence at all.  Those that do may incorrectly
@@ -443,23 +469,25 @@ int sirf_parse(struct gps_device_t *session, unsigned char *buf, int len)
 	     * know what's broken in firmwares before 2.3.1..."
 	     *
 	     * To work around the incomplete implementation of this
-	     * packet in 231, we assume that only the altitude field
-	     * from this packet is valid.
+	     * packet in 231, we used to assume that only the altitude field
+	     * from this packet is valid.  But even this doesn't necessarily
+	     * seem to be the case.  Instead, we do our own computation 
+	     * of geoid separation now.
 	     */
-	    navtype = getw(3);
+	    navtype = (unsigned short)getuw(buf, 3);
 	    session->gpsdata.status = STATUS_NO_FIX;
-	    session->gpsdata.fix.mode = MODE_NO_FIX;
+	    session->gpsdata.newdata.mode = MODE_NO_FIX;
 	    if (navtype & 0x80)
 		session->gpsdata.status = STATUS_DGPS_FIX;
 	    else if ((navtype & 0x07) > 0 && (navtype & 0x07) < 7)
 		session->gpsdata.status = STATUS_FIX;
-	    session->gpsdata.fix.mode = MODE_NO_FIX;
+	    session->gpsdata.newdata.mode = MODE_NO_FIX;
 	    if ((navtype & 0x07) == 4 || (navtype & 0x07) == 6)
-		session->gpsdata.fix.mode = MODE_3D;
+		session->gpsdata.newdata.mode = MODE_3D;
 	    else if (session->gpsdata.status)
-		session->gpsdata.fix.mode = MODE_2D;
+		session->gpsdata.newdata.mode = MODE_2D;
 	    gpsd_report(4, "GNI 0x29: Navtype = 0x%0x, Status = %d, mode = %d\n", 
-			navtype, session->gpsdata.status, session->gpsdata.fix.mode);
+			navtype, session->gpsdata.status, session->gpsdata.newdata.mode);
 	    /*
 	     * UTC is left all zeros in 231 and older firmware versions, 
 	     * and misdocumented in the Protocol Reference (version 1.4).
@@ -472,108 +500,168 @@ int sirf_parse(struct gps_device_t *session, unsigned char *buf, int len)
 	     * UTC second     2               2
 	     *                11              8
 	     */
-	    session->gpsdata.nmea_date.tm_year = getw(11);
-	    session->gpsdata.nmea_date.tm_mon = getb(13)-1;
-	    session->gpsdata.nmea_date.tm_mday = getb(14);
-	    session->gpsdata.nmea_date.tm_hour = getb(15);
-	    session->gpsdata.nmea_date.tm_min = getb(16);
+	    session->gpsdata.nmea_date.tm_year = (int)getuw(buf, 11);
+	    session->gpsdata.nmea_date.tm_mon = (int)getub(buf, 13)-1;
+	    session->gpsdata.nmea_date.tm_mday = (int)getub(buf, 14);
+	    session->gpsdata.nmea_date.tm_hour = (int)getub(buf, 15);
+	    session->gpsdata.nmea_date.tm_min = (int)getub(buf, 16);
 	    session->gpsdata.nmea_date.tm_sec = 0;
-	    session->gpsdata.subseconds = getw(17)*1e-3;
-	    session->gpsdata.fix.time = session->gpsdata.sentence_time
-		= mktime(&session->gpsdata.nmea_date)+session->gpsdata.subseconds;
-	    gpsd_report(5, "MID 41 UTC: %lf\n", session->gpsdata.fix.time);
+	    session->gpsdata.subseconds = getuw(buf, 17)*1e-3;
+	    session->gpsdata.newdata.time = session->gpsdata.sentence_time
+		= (double)mktime(&session->gpsdata.nmea_date)+session->gpsdata.subseconds;
+	    gpsd_report(5, "MID 41 UTC: %lf\n", session->gpsdata.newdata.time);
+#ifdef NTPSHM_ENABLE
+	    if (session->gpsdata.newdata.mode > MODE_NO_FIX && session->gpsdata.nmea_date.tm_year != 0) {
+		if ((session->sirf.time_seen & TIME_SEEN_UTC_1) == 0)
+		    gpsd_report(4, "valid time in message 0x29, seen=0x%02x\n",
+				session->sirf.time_seen);
+		session->sirf.time_seen |= TIME_SEEN_UTC_1;
+		if (IS_HIGHEST_BIT(session->sirf.time_seen,TIME_SEEN_UTC_1))
+		    (void)ntpshm_put(session, session->gpsdata.newdata.time + 0.8);
+	    }
+#endif /* NTPSHM_ENABLE */
 	    /* skip 4 bytes of satellite map */
-	    session->gpsdata.fix.latitude = getl(23)*1e-7;
-	    session->gpsdata.fix.longitude = getl(27)*1e-7;
+	    session->gpsdata.newdata.latitude = getsl(buf, 23)*1e-7;
+	    session->gpsdata.newdata.longitude = getsl(buf, 27)*1e-7;
 	    /* skip 4 bytes of altitude from ellipsoid */
 	    mask = TIME_SET | LATLON_SET | STATUS_SET | MODE_SET;
-	    session->gpsdata.fix.altitude = getl(31)*1e-2;
+	    session->gpsdata.newdata.altitude = getsl(buf, 31)*1e-2;
 	    /* skip 1 byte of map datum */
-	    session->gpsdata.fix.speed = getw(36)*1e-2;
-	    session->gpsdata.fix.track = getw(38)*1e-2;
+	    session->gpsdata.newdata.speed = getsw(buf, 36)*1e-2;
+	    session->gpsdata.newdata.track = getsw(buf, 38)*1e-2;
 	    /* skip 2 bytes of magnetic variation */
-	    session->gpsdata.fix.climb = getw(42)*1e-2;
+	    session->gpsdata.newdata.climb = getsw(buf, 42)*1e-2;
 	    /* HDOP should be available at byte 89, but in 231 it's zero. */
-	    gpsd_binary_fix_dump(session, buf2);
-	    gpsd_report(3, "<= GPS: %s", buf2);
-	    mask |= SPEED_SET | TRACK_SET | CLIMB_SET; 
+	    mask |= SPEED_SET | TRACK_SET | CLIMB_SET | CYCLE_START_SET; 
 	    session->gpsdata.sentence_length = 91;
 	    strcpy(session->gpsdata.tag, "GND");
-	    gpsd_report(4, "Disabling subframe transmission...\n");
-	    sirf_write(session->gpsdata.gps_fd, disablesubframe);
 	}
 	return mask;
 
     case 0x32:		/* SBAS corrections */
 	return 0;
 
+    case 0x34:		/* PPS Time */
+	/*
+	 * Carl Carter from SiRF writes: "We do not output on the
+	 * second (unless you are using message ID 52).  We make
+	 * measurements in the receiver in time with an internal
+	 * counter that is not slaved to GPS time, so the measurements
+	 * are made at a time that wanders around the second.  Then,
+	 * after the measurements are made (all normalized to the same
+	 * point in time) we dispatch the navigation software to make
+	 * a solution, and that solution comes out some 200 to 300 ms
+	 * after the measurement time.  So you may get a message at
+	 * 700 ms after the second that uses measurements time tagged
+	 * 450 ms after the second.  And if some other task jumps up
+	 * and delays things, that message may not come out until 900
+	 * ms after the second.  Things can get out of sync to the
+	 * point that if you try to resolve the GPS time of our 1 PPS
+	 * pulses using the navigation messages, you will find it
+	 * impossible to be consistent.  That is why I added message
+	 * ID 52 to our system -- it is tied to the creation of the 1
+	 * PPS and always comes out right around the top of the
+	 * second."
+	 */
+	mask = 0;
+	gpsd_report(4, "PPS 0x34: Status = 0x%02x\n", getub(buf, 14));
+	if (((int)getub(buf, 14) & 0x07) == 0x07) {	/* valid UTC time? */
+	    session->gpsdata.nmea_date.tm_hour = (int)getub(buf, 1);
+	    session->gpsdata.nmea_date.tm_min = (int)getub(buf, 2);
+	    session->gpsdata.nmea_date.tm_sec = (int)getub(buf, 3);
+	    session->gpsdata.nmea_date.tm_mday = (int)getub(buf, 4);
+	    session->gpsdata.nmea_date.tm_mon = (int)getub(buf, 5) - 1;
+	    session->gpsdata.nmea_date.tm_year = (int)getuw(buf, 6) - 1900;
+	    session->context->leap_seconds = (int)getuw(buf, 8);
+	    session->context->valid |= LEAP_SECOND_VALID;
+#ifdef NTPSHM_ENABLE
+	    if ((session->sirf.time_seen & TIME_SEEN_UTC_2) == 0)
+		gpsd_report(4, "valid time in message 0x34, seen=0x%02x\n",
+				session->sirf.time_seen);
+	    session->sirf.time_seen |= TIME_SEEN_UTC_2;
+	    if (IS_HIGHEST_BIT(session->sirf.time_seen,TIME_SEEN_UTC_2))
+		(void)ntpshm_put(session, session->gpsdata.newdata.time + 0.3);
+#endif /* NTPSHM_ENABLE */
+	    mask |= TIME_SET;
+	}
+	return mask;
+
     case 0x62:		/* uBlox Extended Measured Navigation Data */
 	/* this packet is only sent by uBlox firmware from version 1.32 */
 	mask =	LATLON_SET | ALTITUDE_SET | SPEED_SET | TRACK_SET | CLIMB_SET |
 		STATUS_SET | MODE_SET | HDOP_SET | VDOP_SET | PDOP_SET;
-	session->gpsdata.fix.latitude = getl(1) * RAD_2_DEG * 1e-8; 
-	session->gpsdata.fix.longitude = getl(5) * RAD_2_DEG * 1e-8;
-	session->gpsdata.fix.altitude = getl(9) * 1e-3;
-	session->gpsdata.fix.speed = getl(13) * 1e-3;
-	session->gpsdata.fix.climb = getl(17) * 1e-3;
-	session->gpsdata.fix.track = getl(21) * RAD_2_DEG * 1e-8;
+	session->gpsdata.newdata.latitude = getsl(buf, 1) * RAD_2_DEG * 1e-8; 
+	session->gpsdata.newdata.longitude = getsl(buf, 5) * RAD_2_DEG * 1e-8;
+	session->gpsdata.separation = wgs84_separation(session->gpsdata.newdata.latitude, session->gpsdata.newdata.longitude);
+	session->gpsdata.newdata.altitude = getsl(buf, 9) * 1e-3 - session->gpsdata.separation;
+	session->gpsdata.newdata.speed = getsl(buf, 13) * 1e-3;
+	session->gpsdata.newdata.climb = getsl(buf, 17) * 1e-3;
+	session->gpsdata.newdata.track = getsl(buf, 21) * RAD_2_DEG * 1e-8;
 
-	navtype = getb(25);
+	navtype = (unsigned short)getub(buf, 25);
 	session->gpsdata.status = STATUS_NO_FIX;
-	session->gpsdata.fix.mode = MODE_NO_FIX;
+	session->gpsdata.newdata.mode = MODE_NO_FIX;
 	if (navtype & 0x80)
 	    session->gpsdata.status = STATUS_DGPS_FIX;
 	else if ((navtype & 0x07) > 0 && (navtype & 0x07) < 7)
 	    session->gpsdata.status = STATUS_FIX;
 	if ((navtype & 0x07) == 4 || (navtype & 0x07) == 6)
-	    session->gpsdata.fix.mode = MODE_3D;
+	    session->gpsdata.newdata.mode = MODE_3D;
 	else if (session->gpsdata.status)
-	    session->gpsdata.fix.mode = MODE_2D;
+	    session->gpsdata.newdata.mode = MODE_2D;
 	gpsd_report(4, "EMND 0x62: Navtype = 0x%0x, Status = %d, mode = %d\n", 
-		    navtype, session->gpsdata.status, session->gpsdata.fix.mode);
+		    navtype, session->gpsdata.status, session->gpsdata.newdata.mode);
 
 	if (navtype & 0x40) {		/* UTC corrected timestamp? */
 	    mask |= TIME_SET;
-	    session->gpsdata.nmea_date.tm_year = getw(26) - 1900;
-	    session->gpsdata.nmea_date.tm_mon = getb(28) - 1;
-	    session->gpsdata.nmea_date.tm_mday = getb(29);
-	    session->gpsdata.nmea_date.tm_hour = getb(30);
-	    session->gpsdata.nmea_date.tm_min = getb(31);
+	    session->gpsdata.nmea_date.tm_year = (int)getuw(buf, 26) - 1900;
+	    session->gpsdata.nmea_date.tm_mon = (int)getub(buf, 28) - 1;
+	    session->gpsdata.nmea_date.tm_mday = (int)getub(buf, 29);
+	    session->gpsdata.nmea_date.tm_hour = (int)getub(buf, 30);
+	    session->gpsdata.nmea_date.tm_min = (int)getub(buf, 31);
 	    session->gpsdata.nmea_date.tm_sec = 0;
-	    session->gpsdata.subseconds = ((unsigned short)getw(32))*1e-3;
-	    session->gpsdata.fix.time = session->gpsdata.sentence_time
-		= mkgmtime(&session->gpsdata.nmea_date)+session->gpsdata.subseconds;
+	    session->gpsdata.subseconds = ((unsigned short)getuw(buf, 32))*1e-3;
+	    session->gpsdata.newdata.time = session->gpsdata.sentence_time
+		= (double)mkgmtime(&session->gpsdata.nmea_date)+session->gpsdata.subseconds;
 #ifdef NTPSHM_ENABLE
-	    session->time_seen |= TIME_SEEN_UTC_2;
-	    if (IS_HIGHEST_BIT(session->time_seen,TIME_SEEN_UTC_2))
-		ntpshm_put(session, session->gpsdata.fix.time + 0.75);
+	    if ((session->sirf.time_seen & TIME_SEEN_UTC_2) == 0)
+		gpsd_report(4, "valid time in message 0x62, seen=0x%02x\n",
+				session->sirf.time_seen);
+	    session->sirf.time_seen |= TIME_SEEN_UTC_2;
+	    if (IS_HIGHEST_BIT(session->sirf.time_seen,TIME_SEEN_UTC_2))
+		(void)ntpshm_put(session, session->gpsdata.newdata.time + 0.8);
 #endif /* NTPSHM_ENABLE */
+	    session->context->valid |= LEAP_SECOND_VALID;
 	}
 
-	/* session->gpsdata.gdop = getb(34) / 5.0; */
-	session->gpsdata.pdop = getb(35) / 5.0;
-	session->gpsdata.hdop = getb(36) / 5.0;
-	session->gpsdata.vdop = getb(37) / 5.0;
-	/* session->gpsdata.tdop = getb(38) / 5.0; */
-	session->driverstate |= UBLOX;
-	//gpsd_report(4, "Disabling subframe transmission...\n");
-	//sirf_write(session->gpsdata.gps_fd, disablesubframe);
+	session->gpsdata.gdop = (int)getub(buf, 34) / 5.0;
+	session->gpsdata.pdop = (int)getub(buf, 35) / 5.0;
+	session->gpsdata.hdop = (int)getub(buf, 36) / 5.0;
+	session->gpsdata.vdop = (int)getub(buf, 37) / 5.0;
+	session->gpsdata.tdop = (int)getub(buf, 38) / 5.0;
+	session->sirf.driverstate |= UBLOX;
 	return mask;
 
     case 0xff:		/* Debug messages */
 	buf2[0] = '\0';
-	for (i = 1; i < len; i++)
+	for (i = 1; i < (int)len; i++)
 	    if (isprint(buf[i]))
-		sprintf(buf2+strlen(buf2), "%c", buf[i]);
+		(void)snprintf(buf2+strlen(buf2), 
+			       sizeof(buf2)-strlen(buf2),
+			       "%c", buf[i]);
 	    else
-		sprintf(buf2+strlen(buf2), "\\x%02x", buf[i]);
+		(void)snprintf(buf2+strlen(buf2), 
+			       sizeof(buf2)-strlen(buf2),
+			       "\\x%02x", (unsigned int)buf[i]);
 	gpsd_report(4, "DD  0xff: %s\n", buf2);
 	return 0;
 
     default:
 	buf2[0] = '\0';
-	for (i = 0; i < len; i++)
-	    sprintf(buf2+strlen(buf2), "%02x", buf[i]);
+	for (i = 0; i < (int)len; i++)
+	    (void)snprintf(buf2+strlen(buf2), 
+			   sizeof(buf2)-strlen(buf2),
+			   "%02x", (unsigned int)buf[i]);
 	gpsd_report(3, "Unknown SiRF packet id %d length %d: %s\n", buf[0], len, buf2);
 	return 0;
     }
@@ -581,16 +669,16 @@ int sirf_parse(struct gps_device_t *session, unsigned char *buf, int len)
     return 0;
 }
 
-static int sirfbin_parse_input(struct gps_device_t *session)
+static gps_mask_t sirfbin_parse_input(struct gps_device_t *session)
 {
-    int st;
+    gps_mask_t st;
 
     if (session->packet_type == SIRF_PACKET){
-	st = sirf_parse(session, session->outbuffer+4, session->outbuflen-8);
+	st = sirf_parse(session, session->outbuffer, session->outbuflen);
 	session->gpsdata.driver_mode = 1;
 	return st;
     } else if (session->packet_type == NMEA_PACKET) {
-	st = nmea_parse(session->outbuffer, &session->gpsdata);
+	st = nmea_parse((char *)session->outbuffer, &session->gpsdata);
 	session->gpsdata.driver_mode = 0;
 	return st;
     } else
@@ -602,24 +690,24 @@ static void sirfbin_initializer(struct gps_device_t *session)
 {
     if (session->packet_type == NMEA_PACKET) {
 	gpsd_report(1, "Switching chip mode to SiRF binary.\n");
-	nmea_send(session->gpsdata.gps_fd, 
+	(void)nmea_send(session->gpsdata.gps_fd, 
 		  "$PSRF100,0,%d,8,1,0", session->gpsdata.baudrate);
-	packet_sniff(session);
     }
     /* do this every time*/
     {
-	u_int8_t dgpscontrol[] = {0xa0, 0xa2, 0x00, 0x07,
+	/*@ +charint @*/
+	unsigned char dgpscontrol[] = {0xa0, 0xa2, 0x00, 0x07,
 				 0x85, 0x01, 0x00, 0x00,
 				 0x00, 0x00, 0x00,
 				 0x00, 0x00, 0xb0, 0xb3};
-	u_int8_t sbasparams[] = {0xa0, 0xa2, 0x00, 0x06,
+	unsigned char sbasparams[] = {0xa0, 0xa2, 0x00, 0x06,
 				 0xaa, 0x00, 0x01, 0x00,
 				 0x00, 0x00,
 				 0x00, 0x00, 0xb0, 0xb3};
-	u_int8_t versionprobe[] = {0xa0, 0xa2, 0x00, 0x02,
+	unsigned char versionprobe[] = {0xa0, 0xa2, 0x00, 0x02,
 				 0x84, 0x00,
 				 0x00, 0x00, 0xb0, 0xb3};
-	u_int8_t modecontrol[] = {0xa0, 0xa2, 0x00, 0x0e,
+	unsigned char modecontrol[] = {0xa0, 0xa2, 0x00, 0x0e,
 				  0x88, 
 				  0x00, 0x00,	/* pad bytes */
 				  0x00,		/* degraded mode off */
@@ -632,31 +720,19 @@ static void sirfbin_initializer(struct gps_device_t *session)
 				  0x00,		/* disable dead reckoning */
 				  0x01,		/* enable track smoothing */
 				 0x00, 0x00, 0xb0, 0xb3};
-	u_int8_t enablesubframe[] = {0xa0, 0xa2, 0x00, 0x19,
-				 0x80, 0x00, 0x00, 0x00,
-				 0x00, 0x00, 0x00, 0x00,
-				 0x00, 0x00, 0x00, 0x00,
-				 0x00, 0x00, 0x00, 0x00,
-				 0x00, 0x00, 0x00, 0x00,
-				 0x00, 0x00, 0x00, 0x0C,
-				 0x10,
-				 0x00, 0x00, 0xb0, 0xb3};
+	/*@ -charint @*/
 	gpsd_report(4, "Setting DGPS control to use SBAS...\n");
-	sirf_write(session->gpsdata.gps_fd, dgpscontrol);
+	(void)sirf_write(session->gpsdata.gps_fd, dgpscontrol);
 	gpsd_report(4, "Setting SBAS to auto/integrity mode...\n");
-	sirf_write(session->gpsdata.gps_fd, sbasparams);
+	(void)sirf_write(session->gpsdata.gps_fd, sbasparams);
 	gpsd_report(4, "Probing for firmware version...\n");
-	sirf_write(session->gpsdata.gps_fd, versionprobe);
-	gpsd_report(4, "setting mode...\n");
-	sirf_write(session->gpsdata.gps_fd, modecontrol);
-	if (!(session->context->valid & LEAP_SECOND_VALID)) {
-	    gpsd_report(4, "Enabling subframe transmission...\n");
-	    sirf_write(session->gpsdata.gps_fd, enablesubframe);
-	}
+	(void)sirf_write(session->gpsdata.gps_fd, versionprobe);
+	gpsd_report(4, "Setting mode...\n");
+	(void)sirf_write(session->gpsdata.gps_fd, modecontrol);
     }
 }
 
-static int sirfbin_speed(struct gps_device_t *session, int speed)
+static bool sirfbin_speed(struct gps_device_t *session, speed_t speed)
 {
     return sirf_speed(session->gpsdata.gps_fd, speed);
 }
