@@ -32,20 +32,22 @@ static Widget label_1, label_2, label_3, label_4, label_5;
 static Widget label_6, label_7, label_8, label_9, label_10;
 static GC gc;
 
+/*@ -nullassign @*/
 static XrmOptionDescRec options[] = {
 {"-altunits",  "*altunits",            XrmoptionSepArg,        NULL},
 {"-speedunits","*speedunits",          XrmoptionSepArg,        NULL},
 };
 String fallback_resources[] = {NULL};
+/*@ +nullassign @*/
 
 struct unit_t {
     char *legend;
-    float factor;
+    double factor;
 };
 static struct unit_t speedtable[] = {
-    {"knots",	1},
-    {"mph",		KNOTS_TO_MPH},
-    {"kph",		KNOTS_TO_KPH},
+    {"knots",		1/KNOTS_TO_KPH},
+    {"mph",		MPS_TO_MPH},
+    {"kph",		MPS_TO_KPH},
 }, *speedunits = speedtable;
 static struct unit_t  alttable[] = {
     {"feet",		METERS_TO_FEET},
@@ -57,19 +59,20 @@ static void quit_cb(void)
     exit(0);	/* closes the GPS along with other fds */
 }
 
+/*@ -mustfreefresh -compdef +ignoresigns @*/
 static Pixel get_pixel(Widget w, char *resource_value)
 {
     Colormap colormap;
-    Boolean status;
+    Boolean cstatus;
     XColor exact, color;
 
     colormap = DefaultColormapOfScreen(DefaultScreenOfDisplay(XtDisplay(w)));
-    status = XAllocNamedColor(XtDisplay(w), colormap, resource_value, &color, &exact);
-    if (status == 0) {
-	fprintf(stderr, "Unknown color: %s", resource_value);
+    /*@i@*/cstatus = XAllocNamedColor(XtDisplay(w), colormap, resource_value, &color, &exact);
+    if (cstatus == (Boolean)False) {
+	(void)fprintf(stderr, "Unknown color: %s", resource_value);
 	color.pixel = BlackPixelOfScreen(DefaultScreenOfDisplay(XtDisplay(w)));
     };
-    return (color.pixel);
+    /*@i1@*/return (color.pixel);
 }
 
 static void build_gui(Widget toplevel)
@@ -80,6 +83,7 @@ static void build_gui(Widget toplevel)
     int i;
     XmString string;
 
+    /*@ -immediatetrans -usedef @*/
     /* the root application window */
     XtSetArg(args[0], XmNgeometry, "700x570");
     XtSetArg(args[1], XmNresizePolicy, XmRESIZE_NONE);
@@ -87,8 +91,10 @@ static void build_gui(Widget toplevel)
     XtSetArg(args[3], XmNdeleteResponse, XmDO_NOTHING);
     XtSetArg(args[4], XmNmwmFunctions,
 	     MWM_FUNC_RESIZE | MWM_FUNC_MOVE | MWM_FUNC_MINIMIZE | MWM_FUNC_MAXIMIZE);
+    /*@ +immediatetrans +usedef @*/
     XtSetValues(toplevel, args, 5);
 
+    /*@ -onlytrans @*/
     /* a form to assist with geometry negotiation */
     form = XtVaCreateManagedWidget("form", xmFormWidgetClass, toplevel, NULL);
     /* the left half of the screen */
@@ -115,7 +121,7 @@ static void build_gui(Widget toplevel)
 				     XmNtopWidget, left,
 				     NULL);
     /* satellite location and SNR data panel */
-#define FRAMEHEIGHT	220
+#define FRAMEHEIGHT	234
 #define LEFTSIDE_WIDTH	205
     satellite_list =
       XtVaCreateManagedWidget("satellite_list", xmListWidgetClass, left,
@@ -138,9 +144,11 @@ static void build_gui(Widget toplevel)
     gc = XCreateGC(XtDisplay(satellite_diagram),
 	RootWindowOfScreen(XtScreen(satellite_diagram)), GCForeground, &gcv);
     register_canvas(satellite_diagram, gc);
-    XtAddCallback(satellite_diagram, XmNexposeCallback, (XtPointer)redraw, NULL);
+    /*@i@*/XtAddCallback(satellite_diagram, XmNexposeCallback, (XtPointer)redraw, NULL);
     /* the data display */
+    /*@ -immediatetrans @*/
     XtSetArg(args[0], XmNorientation, XmHORIZONTAL);
+    /*@ +immediatetrans @*/
     rowColumn_11 = XtCreateManagedWidget("time", xmRowColumnWidgetClass, left, args, 1);
 
     rowColumn_12 = XtCreateManagedWidget("latitude", xmRowColumnWidgetClass, left, args, 1);
@@ -165,12 +173,14 @@ static void build_gui(Widget toplevel)
     label_9 = XtCreateManagedWidget("Climb    ", xmLabelWidgetClass, rowColumn_19, args, 0);
     label_10= XtCreateManagedWidget("Status   ", xmLabelWidgetClass, rowColumn_20, args, 0);
 
+    /*@ -immediatetrans @*/
     XtSetArg(args[0], XmNcursorPositionVisible, False);
     XtSetArg(args[1], XmNeditable, False);
     XtSetArg(args[2], XmNmarginHeight, 2);
     XtSetArg(args[3], XmNhighlightThickness, 0);
     XtSetArg(args[4], XmNshadowThickness, 1);
     XtSetArg(args[5], XmNcolumns, 23);
+    /*@ +immediatetrans @*/
     text_1 = XtCreateManagedWidget("text_1", xmTextFieldWidgetClass,
 				   rowColumn_11, args, 6);
     text_2 = XtCreateManagedWidget("text_2", xmTextFieldWidgetClass,
@@ -194,19 +204,22 @@ static void build_gui(Widget toplevel)
 
     quitbutton = XtCreateManagedWidget("Quit",
 			 xmPushButtonWidgetClass, rowColumn_21, args, 0);
-    XtAddCallback(quitbutton, XmNactivateCallback, (XtPointer)quit_cb, NULL);
+    /*@i@*/XtAddCallback(quitbutton, XmNactivateCallback, (XtPointer)quit_cb, NULL);
 
     XtRealizeWidget(toplevel);
     delw = XmInternAtom(XtDisplay(toplevel), "WM_DELETE_WINDOW", False);
-    XmAddWMProtocolCallback(toplevel, delw,
+    /*@ -nullpass @*//* splint 3.1.1 lacks annotated prototype... */
+    /*@i@*/XmAddWMProtocolCallback(toplevel, delw,
 			    (XtCallbackProc)quit_cb, (XtPointer)NULL);
+    /*@ +onlytrans +nullpass @*/
 
     /* create empty list items to be replaced on update */
     string = XmStringCreateSimple(" ");
-    for (i = 0; i < MAXCHANNELS; i++)
-	XmListAddItem(satellite_list, string, i+1);
+    for (i = 0; i <= MAXCHANNELS; i++)
+	XmListAddItem(satellite_list, string, 0);
     XmStringFree(string);
 }
+/*@ +mustfreefresh -ignoresigns +immediatetrans @*/
 
 static void handle_time_out(XtPointer client_data UNUSED,
 			    XtIntervalId *ignored UNUSED)
@@ -223,19 +236,21 @@ static void handle_time_out(XtPointer client_data UNUSED,
 static struct gps_data_t *gpsdata;
 static time_t timer;	/* time of last state change */
 static int state = 0;	/* or MODE_NO_FIX=1, MODE_2D=2, MODE_3D=3 */
-XtAppContext app;
-XtIntervalId timeout;
+static XtAppContext app;
+static XtIntervalId timeout;
 
 static void handle_input(XtPointer client_data UNUSED, int *source UNUSED,
 			 XtInputId *id UNUSED)
 {
     if (gps_poll(gpsdata) < 0) {
-	fprintf(stderr, "Read error on server socket.");
+	(void)fprintf(stderr, "Read error on server socket.");
 	exit(1);
     }
 }
 
-static void update_panel(struct gps_data_t *gpsdata, char *message)
+static void update_panel(struct gps_data_t *gpsdata, 
+			 char *message, 
+			 size_t len UNUSED, int level UNUSED)
 /* runs on each sentence */
 {
     unsigned int i;
@@ -243,7 +258,7 @@ static void update_panel(struct gps_data_t *gpsdata, char *message)
     XmString string[MAXCHANNELS+1];
     char s[128], *sp;
 
-    if (message[0])
+    if (message[0] != '\0')
 	while (isspace(*(sp = message + strlen(message) - 1)))
 	    *sp = '\0';
     XmTextFieldSetString(status, message);
@@ -252,80 +267,80 @@ static void update_panel(struct gps_data_t *gpsdata, char *message)
     if (gpsdata->satellites) {
 	for (i = 0; i < MAXCHANNELS; i++) {
 	    if (i < (unsigned int)gpsdata->satellites) {
-		snprintf(s, sizeof(s), " %3d    %02d    %03d    %02d      %c", 
-			gpsdata->PRN[i],
-			gpsdata->elevation[i], gpsdata->azimuth[i], 
-			gpsdata->ss[i],	gpsdata->used[i] ? 'Y' : 'N'
-		    );
+		(void)snprintf(s, sizeof(s),  
+			       " %3d    %02d    %03d    %02d      %c", 
+			       gpsdata->PRN[i],
+			       gpsdata->elevation[i], gpsdata->azimuth[i], 
+			       gpsdata->ss[i],	gpsdata->used[i] ? 'Y' : 'N');
 	    } else
-		sprintf(s, "                  ");
+		(void)strcpy(s, "                  ");
 	    string[i+1] = XmStringCreateSimple(s);
 	}
-	XmListReplaceItemsPos(satellite_list, string, sizeof(string), 1);
-	for (i = 0; i < sizeof(string)/sizeof(string[0]); i++)
+	XmListReplaceItemsPos(satellite_list, string, (int)sizeof(string), 1);
+	for (i = 0; i < (unsigned int)(sizeof(string)/sizeof(string[0])); i++)
 	    XmStringFree(string[i]);
     }
     /* here are the value fields */
-    if (gpsdata->fix.time != TIME_NOT_VALID)
-	unix_to_iso8601(gpsdata->fix.time, s);
+    if (isnan(gpsdata->fix.time)==0)
+	(void)unix_to_iso8601(gpsdata->fix.time, s, (int)sizeof(s));
     else
-	strcpy(s, "n/a");
+	(void)strcpy(s, "n/a");
     XmTextFieldSetString(text_1, s);
     if (gpsdata->fix.mode >= MODE_2D)
-	sprintf(s, "%f %c", fabsf(gpsdata->fix.latitude), (gpsdata->fix.latitude < 0) ? 'S' : 'N');
+	(void)snprintf(s, sizeof(s), "%lf %c", fabs(gpsdata->fix.latitude), (gpsdata->fix.latitude < 0) ? 'S' : 'N');
     else
-	strcpy(s, "n/a");
+	(void)strcpy(s, "n/a");
     XmTextFieldSetString(text_2, s);
     if (gpsdata->fix.mode >= MODE_2D)
-	sprintf(s, "%f %c", fabsf(gpsdata->fix.longitude), (gpsdata->fix.longitude < 0) ? 'W' : 'E');
+	(void)snprintf(s, sizeof(s), "%lf %c", fabs(gpsdata->fix.longitude), (gpsdata->fix.longitude < 0) ? 'W' : 'E');
     else
-	strcpy(s, "n/a");
+	(void)strcpy(s, "n/a");
     XmTextFieldSetString(text_3, s);
     if (gpsdata->fix.mode == MODE_3D)
-	sprintf(s, "%f %s",gpsdata->fix.altitude*altunits->factor, altunits->legend);
+	(void)snprintf(s, sizeof(s), "%f %s",gpsdata->fix.altitude*altunits->factor, altunits->legend);
     else
-	strcpy(s, "n/a");
+	(void)strcpy(s, "n/a");
     XmTextFieldSetString(text_4, s);
-    if (gpsdata->fix.mode >= MODE_2D && gpsdata->fix.track != TRACK_NOT_VALID)
-	sprintf(s, "%f %s", gpsdata->fix.speed*speedunits->factor, speedunits->legend);
+    if (gpsdata->fix.mode >= MODE_2D && isnan(gpsdata->fix.track)==0)
+	(void)snprintf(s, sizeof(s), "%f %s", gpsdata->fix.speed*speedunits->factor, speedunits->legend);
     else
-	strcpy(s, "n/a");
+	(void)strcpy(s, "n/a");
     XmTextFieldSetString(text_5, s);
-    if (gpsdata->fix.mode >= MODE_2D && gpsdata->fix.track != TRACK_NOT_VALID)
-	sprintf(s, "%f degrees", gpsdata->fix.track);
+    if (gpsdata->fix.mode >= MODE_2D && isnan(gpsdata->fix.track)==0)
+	(void)snprintf(s, sizeof(s), "%f degrees", gpsdata->fix.track);
     else
-	strcpy(s, "n/a");
+	(void)strcpy(s, "n/a");
     XmTextFieldSetString(text_6, s);
-    if (gpsdata->fix.eph != UNCERTAINTY_NOT_VALID)
-	sprintf(s, "%f %s", gpsdata->fix.eph * altunits->factor, altunits->legend);
+    if (isnan(gpsdata->fix.eph)==0)
+	(void)snprintf(s, sizeof(s), "%f %s", gpsdata->fix.eph * altunits->factor, altunits->legend);
     else
-	strcpy(s, "n/a");
+	(void)strcpy(s, "n/a");
     XmTextFieldSetString(text_7, s);
-    if (gpsdata->fix.epv != UNCERTAINTY_NOT_VALID)
-	sprintf(s, "%f %s", gpsdata->fix.epv * altunits->factor, altunits->legend);
+    if (isnan(gpsdata->fix.epv)==0)
+	(void)snprintf(s, sizeof(s), "%f %s", gpsdata->fix.epv * altunits->factor, altunits->legend);
     else
-	strcpy(s, "n/a");
+	(void)strcpy(s, "n/a");
     XmTextFieldSetString(text_8, s);
-    if (gpsdata->fix.mode == MODE_3D)
-	sprintf(s, "%f %s/sec", gpsdata->fix.climb * altunits->factor, altunits->legend);
+    if (gpsdata->fix.mode == MODE_3D && isnan(gpsdata->fix.climb)==0)
+	(void)snprintf(s, sizeof(s), "%f %s/sec", gpsdata->fix.climb * altunits->factor, altunits->legend);
     else
-	strcpy(s, "n/a");
+	(void)strcpy(s, "n/a");
     XmTextFieldSetString(text_9, s);
 
-    if (!gpsdata->online) {
+    if (gpsdata->online == 0) {
 	newstate = 0;
-	sprintf(s, "OFFLINE");
+	(void)snprintf(s, sizeof(s), "OFFLINE");
     } else {
 	newstate = gpsdata->fix.mode;
 	switch (gpsdata->fix.mode) {
 	case MODE_2D:
-	    sprintf(s, "2D %sFIX",(gpsdata->status==STATUS_DGPS_FIX)?"DIFF ":"");
+	    (void)snprintf(s, sizeof(s), "2D %sFIX",(gpsdata->status==STATUS_DGPS_FIX)?"DIFF ":"");
 	    break;
 	case MODE_3D:
-	    sprintf(s, "3D %sFIX",(gpsdata->status==STATUS_DGPS_FIX)?"DIFF ":"");
+	    (void)snprintf(s, sizeof(s), "3D %sFIX",(gpsdata->status==STATUS_DGPS_FIX)?"DIFF ":"");
 	    break;
 	default:
-	    sprintf(s, "NO FIX");
+	    (void)snprintf(s, sizeof(s), "NO FIX");
 	    break;
 	}
     }
@@ -333,7 +348,8 @@ static void update_panel(struct gps_data_t *gpsdata, char *message)
 	timer = time(NULL);
 	state = newstate;
     }
-    sprintf(s + strlen(s), " (%d secs)", (int) (time(NULL) - timer));
+    (void)snprintf(s+strlen(s), sizeof(s)-strlen(s),
+		   " (%d secs)", (int) (time(NULL) - timer));
     XmTextFieldSetString(text_10, s);
     draw_graphics(gpsdata);
 
@@ -346,29 +362,34 @@ static char *get_resource(Widget w, char *name, char *default_value)
   XtResource xtr;
   char *value = NULL;
 
+  /*@ -observertrans -statictrans -immediatetrans -compdestroy @*/
   xtr.resource_name = name;
   xtr.resource_class = "AnyClass";
   xtr.resource_type = XmRString;
-  xtr.resource_size = sizeof(String);
+  xtr.resource_size = (Cardinal)sizeof(String);
   xtr.resource_offset = 0;
   xtr.default_type = XmRImmediate;
   xtr.default_addr = default_value;
   XtGetApplicationResources(w, &value, &xtr, 1, NULL, 0);
   if (value) return value;
-  return default_value;
+  /*@ +observertrans +statictrans +immediatetrans +compdestroy @*/
+  /*@i@*/return default_value;
 }
 
+/*@ -mustfreefresh @*/
 int main(int argc, char *argv[])
 {
     int option;
-    char *colon, *server = NULL, *port = DEFAULT_GPSD_PORT;
+    char *arg = NULL, *colon1, *colon2, *device = NULL, *server = NULL, *port = DEFAULT_GPSD_PORT;
     char *su, *au;
     char *err_str = NULL;
 
+    /*@ -onlytrans */
     toplevel = XtVaAppInitialize(&app, "xgps", 
 			       options, XtNumber(options), 
 			       &argc,argv, fallback_resources,NULL);
 
+    /*@ +onlytrans */
     su = get_resource(toplevel, "speedunits", "mph");
     for (speedunits = speedtable; 
 	 speedunits < speedtable + sizeof(speedtable)/sizeof(speedtable[0]);
@@ -376,7 +397,7 @@ int main(int argc, char *argv[])
 	if (strcmp(speedunits->legend, su) == 0)
 	    goto speedunits_ok;
     speedunits = speedtable;
-    fprintf(stderr, "xgps: unknown speed unit, defaulting to %s\n", speedunits->legend);
+    (void)fprintf(stderr, "xgps: unknown speed unit, defaulting to %s\n", speedunits->legend);
 speedunits_ok:;
 
     au = get_resource(toplevel, "altunits",   "feet");
@@ -386,29 +407,44 @@ speedunits_ok:;
 	if (strcmp(altunits->legend, au) == 0)
 	    goto altunits_ok;
     altunits = alttable;
-    fprintf(stderr, "xgps: unknown altitude unit, defaulting to %s\n", altunits->legend);
+    (void)fprintf(stderr, "xgps: unknown altitude unit, defaulting to %s\n", altunits->legend);
 altunits_ok:;
 
-    while ((option = getopt(argc, argv, "?hv")) != -1) {
+    while ((option = getopt(argc, argv, "hv")) != -1) {
 	switch (option) {
 	case 'v':
-	    printf("xgps %s\n", VERSION);
+	    (void)printf("xgps %s\n", VERSION);
 	    exit(0);
-	case 'h': case '?': default:
-	    fputs("usage:  xgps [-?hv] [-speedunits {mph,kph,knots}] [-altunits {ft,meters}] [server[:port]]\n", stderr);
+	case 'h': default:
+	    (void)fputs("usage:  xgps [-hv] [-speedunits {mph,kph,knots}] [-altunits {ft,meters}] [server[:port:[device]]]\n", stderr);
 	    exit(1);
 	}
     }
+    /*@ -branchstate @*/
     if (optind < argc) {
-	server = strdup(argv[optind]);
-	colon = strchr(server, ':');
-	if (colon != NULL) {
-	    server[colon - server] = '\0';
-	    port = colon + 1;
+	arg = strdup(argv[optind]);
+	/*@i@*/colon1 = strchr(arg, ':');
+	server = arg;
+	if (colon1 != NULL) {
+	    if (colon1 == arg)
+		server = NULL;
+	    else
+		*colon1 = '\0';
+	    port = colon1 + 1;
+	    colon2 = strchr(port, ':');
+	    if (colon2 != NULL) {
+		if (colon2 == port)
+		    port = NULL;
+	        else
+		    *colon2 = '\0';
+		device = colon2 + 1;
+	    }
 	}
+	colon1 = colon2 = NULL;
     }
+    /*@ +branchstate @*/
 
-    gpsdata = gps_open(server, port);
+    /*@i@*/gpsdata = gps_open(server, port);
     if (!gpsdata) {
 	switch ( errno ) {
 	case NL_NOSERVICE: 	err_str = "can't get service entry"; break;
@@ -419,8 +455,9 @@ altunits_ok:;
 	case NL_NOCONNECT: 	err_str = "can't connect to host"; break;
 	default:             	err_str = "Unknown"; break;
 	}
-	fprintf( stderr, "xgps: no gpsd running or network error: %d, %s\n"
-		, errno, err_str);
+	(void)fprintf( stderr, 
+		       "xgps: no gpsd running or network error: %d, %s\n", 
+		       errno, err_str);
 	exit(2);
     }
 
@@ -430,12 +467,27 @@ altunits_ok:;
     timer = time(NULL);
 
     gps_set_raw_hook(gpsdata, update_panel);
-    gps_query(gpsdata, "w+x\n");
 
-    XtAppAddInput(app, gpsdata->gps_fd, 
+    if (device) {
+	char *channelcmd = (char *)malloc(strlen(device)+3);
+
+	if (channelcmd) {
+	    /*@i1@*/(void)strcpy(channelcmd, "F=");
+	    (void)strcpy(channelcmd+2, device);
+	    (void)gps_query(gpsdata, channelcmd);
+	    (void)free(channelcmd);
+	}
+    }
+	
+    (void)gps_query(gpsdata, "w+x\n");
+
+    (void)XtAppAddInput(app, gpsdata->gps_fd, 
 		  (XtPointer)XtInputReadMask, handle_input, NULL);
-    XtAppMainLoop(app);
+    (void)XtAppMainLoop(app);
 
-    gps_close(gpsdata);
+    (void)gps_close(gpsdata);
+    if (arg != NULL)
+	(void)free(arg);
     return 0;
 }
+/*@ +mustfreefresh @*/
